/*******************************************************************************

                      LEGO for POV-Ray 3.5
                     (C) Ernst Martin Witte

   This file is in no way related to the LEGO(tm) Group.
   It is provided for private non-commercial use only.


   Objects:  Bush
             HeadBush
             HalfBush
             HalfHeadBush
             Crank
             Cone1x1x3


 *******************************************************************************/

#ifndef (__LEGO_BUSH_INC)

  #declare __LEGO_BUSH_INC = 1;
  
  #include "roundcylinder.inc"
  #include "invroundcylinder.inc"
  #include "roundbox.inc"
  #include "invroundbox.inc"
  #include "inv_round_edge.inc"


  #declare Bush = difference {
    union {
      difference {
	union {
	  cylinder { <0, 0, 0> <LEGO_LENGTH, 0, 0> LEGO_BUSH_RADIUS }
	  object { invround_coneedgeX(LEGO_BUSH_RADIUS, 90, -90, LEGO_CORNER_SPACE/2)
	    rotate -z*90 translate x*LEGO_BUSH_END_WIDTH }
	  object { invround_coneedgeX(LEGO_BUSH_RADIUS, 90, -90, LEGO_CORNER_SPACE/2)
	    rotate z*90 translate x*(LEGO_LENGTH-LEGO_BUSH_END_WIDTH) }
	}
	object { invroundbox(LEGO_BUSH_RADIUS*2, LEGO_LENGTH-2*LEGO_BUSH_END_WIDTH, LEGO_GAP_WIDTH)
	  rotate -z*90
	  translate <LEGO_BUSH_END_WIDTH, LEGO_BUSH_RADIUS, -LEGO_GAP_WIDTH/2>
	}
      }
      object { roundcylinder(LEGO_BUSH_END_WIDTH, LEGO_BUSH_END_RADIUS) }
      object { roundcylinder(LEGO_BUSH_END_WIDTH, LEGO_BUSH_END_RADIUS) scale <-1,1,1> translate x*LEGO_LENGTH }
    }
    #local iii = 0;
    #while (iii < 4)
      object { invroundcylinder(LEGO_BUSH_END_WIDTH, LEGO_KNOB_RADIUS)
	translate <0, LEGO_LENGTH/2, LEGO_LENGTH/2>
	rotate x*90*iii }
      #local iii=iii+1;
    #end

    object { __IAxle scale <1.1, 1, 1>}
    object { __INV_ROUND_AXLE translate x*LEGO_LENGTH }
    object { __INV_ROUND_AXLE rotate y*180 }
    
    texture { LegoGray }
    bounded_by { cylinder { <0, 0, 0> <LEGO_LENGTH, 0, 0> LEGO_BUSH_END_RADIUS }}
  }



  
  
  
  #declare HeadBush = difference {
    union {
      difference {
	union {
	  cylinder { <0, 0, 0> <LEGO_LENGTH, 0, 0> LEGO_BUSH_RADIUS }
	  object { invround_coneedgeX(LEGO_BUSH_RADIUS, 90, -90, LEGO_CORNER_SPACE/2)
	    rotate -z*90 translate x*LEGO_BUSH_END_WIDTH }
	  object { invround_coneedgeX(LEGO_BUSH_RADIUS, 90, -90, LEGO_CORNER_SPACE/2)
	    rotate z*90 translate x*(LEGO_LENGTH-LEGO_BUSH_END_WIDTH) }
	}
	object { invroundbox(LEGO_BUSH_RADIUS*2, LEGO_LENGTH-2*LEGO_BUSH_END_WIDTH, LEGO_GAP_WIDTH)
	  rotate -z*90
	  translate <LEGO_BUSH_END_WIDTH, LEGO_BUSH_RADIUS, -LEGO_GAP_WIDTH/2> 
	  rotate x*90
	}
	
      }
      object { roundcylinder(LEGO_BUSH_END_WIDTH, LEGO_BUSH_END_RADIUS) }
      object { h_roundcylinderX(LEGO_LENGTH/2+LEGO_BUSH_END_WIDTH, LEGO_LENGTH/2, LEGO_KNOB_CORNER_SPACE)
	translate x*(LEGO_LENGTH-LEGO_BUSH_END_WIDTH) }
      
      
      intersection {
	object { roundcylinder(LEGO_LENGTH, 1.7+LEGO_TUBE_INLAY_RADIUS)
	  rotate z*90 translate <LEGO_LENGTH*1.5, -LEGO_LENGTH/2, 0> }
	object { roundbox (LEGO_LENGTH+LEGO_KNOB_RADIUS+EXTRA, LEGO_LENGTH+2*EXTRA, LEGO_LENGTH+2*EXTRA)
	  translate <LEGO_LENGTH/2, -LEGO_LENGTH/2-EXTRA, -LEGO_LENGTH/2-EXTRA> }
      }
      
      
      object { roundcylinder(LEGO_LENGTH, LEGO_TUBE_RADIUS)
	rotate z*90 translate <LEGO_LENGTH*1.5, -LEGO_LENGTH/2, 0> }
      
      object { roundcylinderX(LEGO_LENGTH+2*LEGO_KNOB_HEIGHT, LEGO_KNOB_RADIUS, LEGO_KNOB_CORNER_SPACE)
	rotate -y*90 translate <LEGO_LENGTH*1.5, 0, -LEGO_LENGTH/2-LEGO_KNOB_HEIGHT> }
      
    }
    
    object { __IAxle }
    object { h_invroundcylinder(LEGO_BUSH_END_WIDTH, LEGO_KNOB_RADIUS) }
    
    intersection {
      cylinder { <0, 0, 0> <LEGO_BUSH_END_WIDTH*3, 0, 0> LEGO_KNOB_RADIUS }
      object { __INV_ROUND_AXLE scale <-1,1,1> translate x*LEGO_BUSH_END_WIDTH }
    }
      
    #local iii = 0;
    #while (iii < 4)
      object { invroundcylinder(LEGO_BUSH_END_WIDTH, LEGO_KNOB_RADIUS)
	translate <0, LEGO_LENGTH/2, LEGO_LENGTH/2> rotate x*iii*90 }
      #local iii=iii+1;
    #end
    
    object {
      LEGO_INV_TUBE
      translate <0, LEGO_TUBE_RADIUS-3*LEGO_HEIGHT, -LEGO_LENGTH/2>
      rotate x*90
      translate 1.5*LEGO_LENGTH*x
    }
    
    texture { LegoGray }
    bounded_by { box { <0, -LEGO_LENGTH/2, -LEGO_LENGTH/2-LEGO_KNOB_HEIGHT> <2*LEGO_LENGTH, LEGO_LENGTH/2, LEGO_LENGTH/2+LEGO_KNOB_HEIGHT> }}
  }






  #declare HalfBush = difference {
    union {
      difference {
	union {
	  cylinder { <0, 0, 0> <LEGO_LENGTH/2-LEGO_BUSH_END_WIDTH+EXTRA, 0, 0> LEGO_BUSH_RADIUS }
	  object { invround_coneedgeX(LEGO_BUSH_RADIUS, 90, -90, LEGO_CORNER_SPACE/2)
	    rotate -z*90 translate x*LEGO_BUSH_END_WIDTH }
	  object { invround_coneedgeX(LEGO_BUSH_RADIUS, 90, -90, LEGO_CORNER_SPACE/2)
	    rotate  z*90 translate x*(LEGO_LENGTH/2-LEGO_BUSH_END_WIDTH) }
	  object { roundcylinder(LEGO_BUSH_END_WIDTH, LEGO_BUSH_END_RADIUS) }
	  object { roundcylinder(LEGO_BUSH_END_WIDTH-LEGO_WALL_WIDTH/2, LEGO_BUSH_END_RADIUS) scale <-1,1,1> translate x*(LEGO_LENGTH/2-LEGO_WALL_WIDTH/2) }
	}
	
	object { __INV_ROUND_AXLE scale <-1,1,1> }
	object { __INV_ROUND_AXLE translate x*(LEGO_LENGTH/2-LEGO_WALL_WIDTH/2) }
      }
      
      difference {
	union {
	  object { h_roundtube(LEGO_WALL_WIDTH/2, LEGO_BUSH_END_RADIUS, LEGO_KNOB_RADIUS)  rotate y*180 translate x*LEGO_LENGTH/2 }
	  difference {
	    cylinder { <LEGO_LENGTH/2-LEGO_WALL_WIDTH/2-LEGO_CORNER_SPACE*1.5-EXTRA,0,0> <LEGO_LENGTH/2-LEGO_WALL_WIDTH/2+EXTRA,0,0> LEGO_BUSH_END_RADIUS }
	    cylinder { <0,0,0> <LEGO_LENGTH,0,0> LEGO_KNOB_RADIUS }
	  }
	}
	intersection {
	  object { __INV_ROUND_AXLE translate x*(LEGO_LENGTH/2-LEGO_WALL_WIDTH/2) }
	  cylinder { x*LEGO_LENGTH/4 x*LEGO_LENGTH/2 LEGO_KNOB_RADIUS }
	}
	object {
	  __IKranzX(LEGO_WALL_WIDTH/2+LEGO_CORNER_SPACE+EXTRA) 
	  rotate -z*90 
	  translate x*(LEGO_LENGTH/2 - LEGO_WALL_WIDTH/2-LEGO_CORNER_SPACE-EXTRA) 
	}
	#local i=0;
	#while (i<16)
	  object { inv_roundedgeX(LEGO_WALL_WIDTH, 90, LEGO_CORNER_SPACE/2)
	    translate <LEGO_LENGTH/2-LEGO_WALL_WIDTH/2-LEGO_CORNER_SPACE, LEGO_BUSH_END_RADIUS> 
	    rotate x*(360/16*i+360/32*0.025) }
	  object { inv_roundedgeX(LEGO_WALL_WIDTH, 90, LEGO_CORNER_SPACE/2)
	    translate <LEGO_LENGTH/2-LEGO_WALL_WIDTH/2-LEGO_CORNER_SPACE, LEGO_BUSH_END_RADIUS> 
	    scale <1,1,-1>
	    rotate x*(360/16*i-360/32*1.025) }
	  
	  object { inv_roundedgeX(LEGO_WALL_WIDTH, 90, LEGO_CORNER_SPACE/2)
	    translate <LEGO_LENGTH/2-LEGO_WALL_WIDTH/2-LEGO_CORNER_SPACE, -LEGO_KNOB_RADIUS> 
	    rotate x*(360/16*i+360/32*0.975) }
	  object { inv_roundedgeX(LEGO_WALL_WIDTH, 90, LEGO_CORNER_SPACE/2)
	    translate <LEGO_LENGTH/2-LEGO_WALL_WIDTH/2-LEGO_CORNER_SPACE, -LEGO_KNOB_RADIUS> 
	    scale <1,1,-1>
	    rotate x*(360/16*i+360/32*0.025) }
	  #declare i=i+1;
	#end
	
	bounded_by {
	  cylinder { 0 x*(LEGO_LENGTH/2+EXTRA) LEGO_BUSH_END_RADIUS+EXTRA }
	}
      }
    }

    object { __IAxle }
    
    texture { LegoGray }
    bounded_by { cylinder { <0, 0, 0> <LEGO_LENGTH/2, 0, 0>, LEGO_BUSH_END_RADIUS }}
  }




  #declare HalfHeadBush = difference {
    union {
      difference {
	union {
	  
	  difference {
	    union {
	      cylinder { <LEGO_WALL_WIDTH/2, 0, 0> <LEGO_LENGTH, 0, 0> LEGO_BUSH_RADIUS }
	      
	      object { invround_coneedgeX(LEGO_BUSH_RADIUS, 90, -90, LEGO_CORNER_SPACE/2)
		rotate -z*90 translate x*LEGO_BUSH_END_WIDTH }
	      object { invround_coneedgeX(LEGO_BUSH_RADIUS, 90, -90, LEGO_CORNER_SPACE/2)
		rotate z*90 translate x*(LEGO_LENGTH-LEGO_BUSH_END_WIDTH) }
	      object { roundcylinder(LEGO_BUSH_END_WIDTH, LEGO_BUSH_END_RADIUS) scale <-1,1,1> translate x*LEGO_LENGTH }
	      
	  
	      difference {
		object { roundcylinder(LEGO_BUSH_END_WIDTH-LEGO_WALL_WIDTH/2, LEGO_BUSH_END_RADIUS) 
		  translate x*LEGO_WALL_WIDTH/2 }
		object { invroundcylinder(LEGO_BUSH_END_WIDTH-LEGO_WALL_WIDTH/2, LEGO_KNOB_RADIUS) translate <LEGO_WALL_WIDTH/2,  LEGO_LENGTH/2,  LEGO_LENGTH/2> }	  
		object { invroundcylinder(LEGO_BUSH_END_WIDTH-LEGO_WALL_WIDTH/2, LEGO_KNOB_RADIUS) translate <LEGO_WALL_WIDTH/2,  LEGO_LENGTH/2, -LEGO_LENGTH/2> }	  
		object { invroundcylinder(LEGO_BUSH_END_WIDTH-LEGO_WALL_WIDTH/2, LEGO_KNOB_RADIUS) translate <LEGO_WALL_WIDTH/2, -LEGO_LENGTH/2,  LEGO_LENGTH/2> }	  
		object { invroundcylinder(LEGO_BUSH_END_WIDTH-LEGO_WALL_WIDTH/2, LEGO_KNOB_RADIUS) translate <LEGO_WALL_WIDTH/2, -LEGO_LENGTH/2, -LEGO_LENGTH/2> }
	      }    
	    }
	    object { __INV_ROUND_AXLE rotate y*180 translate x*LEGO_WALL_WIDTH/2 }
	  }

	  difference {
	    intersection {
	      cylinder { <LEGO_LENGTH-LEGO_BUSH_END_WIDTH/2, 0, 0> <LEGO_LENGTH*1.5-LEGO_WALL_WIDTH/4, 0, 0> LEGO_BUSH_END_RADIUS }
	      object { roundbox(LEGO_LENGTH, LEGO_BUSH_END_RADIUS, 2*LEGO_BUSH_END_RADIUS)
		translate <LEGO_LENGTH-LEGO_BUSH_END_WIDTH, -LEGO_BUSH_END_RADIUS, -LEGO_BUSH_END_RADIUS> }
	    }
	    object { h_invroundcylinder(LEGO_WALL_WIDTH/4, LEGO_BUSH_END_RADIUS)
	      rotate -z*90 translate x*1.5*LEGO_LENGTH }
	  }
	  
	  object { roundcylinder(LEGO_BUSH_END_RADIUS-LEGO_WALL_WIDTH/4, LEGO_BUSH_END_RADIUS) 
	    rotate -z*90 translate <1.5*LEGO_LENGTH, -LEGO_WALL_WIDTH/4,0> }
	
	
	  difference {
	    union {
	      object { h_roundtube(LEGO_WALL_WIDTH/2, LEGO_BUSH_END_RADIUS, LEGO_KNOB_RADIUS) }
	      difference {
		cylinder { <LEGO_WALL_WIDTH/2-EXTRA,0,0> <LEGO_WALL_WIDTH/2+LEGO_CORNER_SPACE*1.5+EXTRA,0,0> LEGO_BUSH_END_RADIUS }
		cylinder { <0,0,0> <LEGO_LENGTH,0,0> LEGO_KNOB_RADIUS }
	      }
	    }
	    intersection {
	      object { __INV_ROUND_AXLE rotate y*180 translate x*LEGO_WALL_WIDTH/2 }
	      cylinder { 0 x*LEGO_LENGTH/4 LEGO_KNOB_RADIUS }
	    }
	    object {
	      __IKranzX(LEGO_WALL_WIDTH/2+LEGO_CORNER_SPACE+EXTRA) 
	      rotate z*90
	      translate x*(LEGO_WALL_WIDTH/2+LEGO_CORNER_SPACE)
	    }
	    object { h_invroundcylinder(LEGO_WALL_WIDTH, LEGO_KNOB_RADIUS) translate ( y+z)*LEGO_LENGTH/2 }
	    object { h_invroundcylinder(LEGO_WALL_WIDTH, LEGO_KNOB_RADIUS) translate ( y-z)*LEGO_LENGTH/2 }
	    object { h_invroundcylinder(LEGO_WALL_WIDTH, LEGO_KNOB_RADIUS) translate (-y+z)*LEGO_LENGTH/2 }
	    object { h_invroundcylinder(LEGO_WALL_WIDTH, LEGO_KNOB_RADIUS) translate (-y-z)*LEGO_LENGTH/2 }
	    #local i=0;
	    #while (i<16)
	      object { inv_roundedgeX(LEGO_WALL_WIDTH, 90, LEGO_CORNER_SPACE/2)
		translate <LEGO_LENGTH/2-LEGO_WALL_WIDTH/2-LEGO_CORNER_SPACE, -LEGO_KNOB_RADIUS> 
		rotate x*(360/16*i+360/32*0.975) }
	      object { inv_roundedgeX(LEGO_WALL_WIDTH, 90, LEGO_CORNER_SPACE/2)
		translate <LEGO_LENGTH/2-LEGO_WALL_WIDTH/2-LEGO_CORNER_SPACE, -LEGO_KNOB_RADIUS> 
		scale <1,1,-1>
		rotate x*(360/16*i+360/32*0.025) }
	      #declare i=i+1;
	    #end
	
	    bounded_by {
	      cylinder { 0 x*(LEGO_LENGTH/2+EXTRA) LEGO_BUSH_END_RADIUS+EXTRA }
	    }
	  }


	}
	
	object { h_invroundcylinder(0.5, (LEGO_KNOB_RADIUS+LEGO_BUSH_END_RADIUS)/2)
	  rotate -z*90 translate <1.5*LEGO_LENGTH, -LEGO_WALL_WIDTH/4> }
	object { invroundcylinder(LEGO_BUSH_END_RADIUS-LEGO_WALL_WIDTH/2-0.5-LEGO_WALL_WIDTH/4, LEGO_KNOB_RADIUS)
	  rotate z*90 translate <1.5*LEGO_LENGTH, -LEGO_BUSH_END_RADIUS+LEGO_WALL_WIDTH/2> }
	object { h_invroundcylinder(LEGO_WALL_WIDTH/2, LEGO_TUBE_INLAY_RADIUS)
	  rotate z*90 translate <1.5*LEGO_LENGTH, -LEGO_BUSH_END_RADIUS> }
	
	intersection {
	  object { __INV_ROUND_AXLE }
	  box { <-2*LEGO_CORNER_SPACE, EXTRA, -LEGO_KNOB_RADIUS-1> <2*LEGO_CORNER_SPACE, LEGO_KNOB_RADIUS+1, LEGO_KNOB_RADIUS+1> }
	  translate x*LEGO_LENGTH
	}
	
      }
      
      difference {
	object { h_roundcylinder(LEGO_WALL_WIDTH/2+LEGO_CORNER_SPACE, LEGO_BUSH_END_RADIUS)
	  rotate -z*90 translate y*LEGO_WALL_WIDTH/4 }
	object {
	  __IKranz
	  translate y*(-LEGO_HEIGHT+LEGO_WALL_WIDTH/4)
	}
	object { h_invroundcylinder(LEGO_WALL_WIDTH, (LEGO_KNOB_RADIUS+LEGO_BUSH_END_RADIUS)/2)
	  rotate -z*90 translate y*LEGO_WALL_WIDTH/4 }
             
	#local i=0;
	#while (i<16)
	  object { inv_roundedgeX(LEGO_WALL_WIDTH, 90, LEGO_CORNER_SPACE/2)
	    rotate z*90
	    translate <-LEGO_BUSH_END_RADIUS, -LEGO_WALL_WIDTH/2> 
	    rotate y*(360/16*i+360/32*0.025) }
	  object { inv_roundedgeX(LEGO_WALL_WIDTH, 90, LEGO_CORNER_SPACE/2)
	    rotate z*90
	    scale <1,1,-1>
	    translate <-LEGO_BUSH_END_RADIUS, -LEGO_WALL_WIDTH/2> 
	    rotate y*(360/16*i-360/32*1.025) }
	  
	  object { inv_roundedgeX(LEGO_WALL_WIDTH, 90, LEGO_CORNER_SPACE/2)
	    rotate z*90
	    translate <(LEGO_KNOB_RADIUS+LEGO_BUSH_END_RADIUS)/2, -LEGO_WALL_WIDTH/2> 
	    rotate y*(360/16*i+360/32*0.975) }
	  object { inv_roundedgeX(LEGO_WALL_WIDTH, 90, LEGO_CORNER_SPACE/2)
	    rotate z*90
	    scale <1,1,-1>
	    translate <(LEGO_KNOB_RADIUS+LEGO_BUSH_END_RADIUS)/2, -LEGO_WALL_WIDTH/2> 
	    rotate y*(360/16*i+360/32*0.025) }
	  #declare i=i+1;
	#end
	
	bounded_by { cylinder { <0, -LEGO_WALL_WIDTH/2, 0> <0, LEGO_WALL_WIDTH/2, 0> LEGO_BUSH_END_RADIUS+1}}

	translate <1.5*LEGO_LENGTH, 0, 0>
      }

    }
    
    object { __IAxle }
  
    texture { LegoGray }
    bounded_by { 
      box { <0, -LEGO_BUSH_END_RADIUS, -LEGO_BUSH_END_RADIUS> <2*LEGO_LENGTH, LEGO_BUSH_END_RADIUS, LEGO_BUSH_END_RADIUS> }
    }
  }
  


#declare Crank = difference {
    union {
        difference {
            box { <-LEGO_LENGTH/2, -LEGO_LENGTH/2, -LEGO_BUSH_END_RADIUS>
                  < LEGO_LENGTH/2,  LEGO_LENGTH/2,  LEGO_BUSH_END_RADIUS>
            }

            #local a = -1;
            #while (a < 2)
                #local b = -1;
                #while (b < 2)
                    object {
                        INV_CORNER_CYLINDER    translate -x     rotate -z*45
                        translate <-LEGO_LENGTH/2-LEGO_BUSH_END_RADIUS+2*LEGO_HEIGHT, LEGO_LENGTH/2, LEGO_BUSH_END_RADIUS>
                        scale <a, a, b>
                    }
                    object {
                        INV_CORNER_CYLINDER    translate -x      rotate -x*90     rotate -z*45
                        translate <-LEGO_LENGTH/2-LEGO_BUSH_END_RADIUS+2*LEGO_HEIGHT, LEGO_LENGTH/2, LEGO_KNOB_RADIUS>
                        scale <a, a, b>
                    }
                    object {
                        INV_CORNER_CYLINDER    translate -x*LEGO_LENGTH
                        translate <-LEGO_LENGTH/2-LEGO_BUSH_END_RADIUS+2*LEGO_HEIGHT, LEGO_LENGTH/2, LEGO_BUSH_END_RADIUS>
                        scale <a, a, b>
                    }
                    #local b = b+2;
                #end
                box { <-1, 0, -LEGO_LENGTH/2>
                      <LEGO_LENGTH, LEGO_LENGTH, LEGO_LENGTH/2>
                      rotate -z*45
                      translate <-LEGO_LENGTH/2-LEGO_BUSH_END_RADIUS+2*LEGO_HEIGHT, LEGO_LENGTH/2, 0>
                      scale <a, a, 1>
                }
                box { <-1, 0, -LEGO_KNOB_RADIUS>
                      <LEGO_LENGTH, LEGO_LENGTH, LEGO_KNOB_RADIUS>
                      rotate -z*45
                      translate <-LEGO_LENGTH/2-LEGO_BUSH_END_RADIUS+2*LEGO_HEIGHT - 0.6*sqrt(2), +LEGO_LENGTH/2, 0>
                      scale <a, a, 1>
                }

                object {
                    INV_CORNER_CYLINDER135
                    translate -x*LEGO_LENGTH
                    rotate y*90
                    translate <-LEGO_LENGTH/2-LEGO_BUSH_END_RADIUS+2*LEGO_HEIGHT, LEGO_LENGTH/2, 0>
                    scale <a, a, 1>
                }

                #local a = a+2;
            #end
        }

        #local a = -1;
        #while (a < 2)
            union {
                difference {
                    box {
                        <LEGO_LENGTH/2, LEGO_WALL_WIDTH/2, -LEGO_BUSH_END_RADIUS>
                        <LEGO_LENGTH/2+LEGO_BUSH_END_RADIUS-2*LEGO_HEIGHT, -LEGO_LENGTH/4, LEGO_BUSH_END_RADIUS>
                    }
                    object {
                        INV_CORNER_CYLINDER
                        translate <0, LEGO_WALL_WIDTH/2, LEGO_BUSH_END_RADIUS>
                    }
                    object {
                        INV_CORNER_CYLINDER
                        translate <0, LEGO_WALL_WIDTH/2, LEGO_BUSH_END_RADIUS>
                        scale <1,1,-1>
                    }
                }
                cylinder {
                    <LEGO_LENGTH/2, LEGO_WALL_WIDTH/2, 0>
                    <LEGO_LENGTH/2, -LEGO_LENGTH/2, 0>
                    LEGO_BUSH_END_RADIUS-LEGO_CORNER_SPACE
                }
                cylinder {
                    <LEGO_LENGTH/2, LEGO_WALL_WIDTH/2-LEGO_CORNER_SPACE, 0>
                    <LEGO_LENGTH/2, -LEGO_LENGTH/2+LEGO_CORNER_SPACE, 0>
                    LEGO_BUSH_END_RADIUS
                }
                torus {
                    LEGO_BUSH_END_RADIUS-LEGO_CORNER_SPACE
                    LEGO_CORNER_SPACE
                    translate <LEGO_LENGTH/2, LEGO_WALL_WIDTH/2-LEGO_CORNER_SPACE, 0>
                }
                torus {
                    LEGO_BUSH_END_RADIUS-LEGO_CORNER_SPACE
                    LEGO_CORNER_SPACE
                    translate <LEGO_LENGTH/2, -LEGO_LENGTH/2+LEGO_CORNER_SPACE, 0>
                }
                scale <a,a,1>
            }
            #local a = a+2;
        #end
    }
    #local a = -LEGO_LENGTH/2-LEGO_BUSH_END_RADIUS+2*LEGO_HEIGHT+LEGO_LENGTH/2-LEGO_WALL_WIDTH/2-0.6*sqrt(2);
    box { <-a, -LEGO_LENGTH, -LEGO_GAP_WIDTH/2-EXTRA>
          < a,  LEGO_LENGTH, LEGO_GAP_WIDTH/2+EXTRA> }
    box { <-LEGO_LENGTH/2, 0, -LEGO_GAP_WIDTH/2>
          < 0,  LEGO_LENGTH, LEGO_GAP_WIDTH/2> }
    box { < LEGO_LENGTH/2, 0, -LEGO_GAP_WIDTH/2>
          < 0,  -LEGO_LENGTH, LEGO_GAP_WIDTH/2> }

    object { __IAxle    rotate z*90   translate <LEGO_LENGTH/2, -LEGO_LENGTH, 0> }
    object { __IAxle    rotate -z*90  translate <-LEGO_LENGTH/2, LEGO_LENGTH, 0> }
    object { __INV_ROUND_AXLE  rotate z*90 translate <-LEGO_LENGTH/2, LEGO_LENGTH/2, 0> }
    object { __INV_ROUND_AXLE  rotate z*-90  translate <LEGO_LENGTH/2, -LEGO_LENGTH/2, 0> }


    bounded_by { box { <-1, -0.5, -0.5> <1, 0.5, 0.5> scale LEGO_LENGTH }}

    translate <LEGO_LENGTH/2, LEGO_LENGTH/2, 0>

    texture { LegoGray }
}



#declare Cone1x1x3 = union {
    #local PHI=atan2 (2*LEGO_HEIGHT+LEGO_CEILING_HEIGHT, LEGO_LENGTH/2-LEGO_KNOB_RADIUS);
    #local RR=LEGO_CORNER_SPACE/tan (PHI/2);

    difference {
        union {
            cylinder { <0, 0, 0> 
                       <0, LEGO_HEIGHT-LEGO_CEILING_HEIGHT + EXTRA, 0> 
                       LEGO_HEIGHT - LEGO_CORNER_SPACE }
            cylinder { <0, LEGO_CORNER_SPACE, 0> 
                       <0, LEGO_HEIGHT-LEGO_CEILING_HEIGHT+EXTRA, 0> 
                       LEGO_HEIGHT }
            torus { LEGO_HEIGHT-LEGO_CORNER_SPACE 
                    LEGO_CORNER_SPACE 
                    translate y*LEGO_CORNER_SPACE }

            cone { <0, LEGO_HEIGHT-LEGO_CEILING_HEIGHT, 0> LEGO_LENGTH/2   
                   <0, 3*LEGO_HEIGHT, 0> LEGO_KNOB_RADIUS }

            cylinder { <0, 3*LEGO_HEIGHT-EXTRA, 0> 
                       <0, 3*LEGO_HEIGHT+LEGO_KNOB_HEIGHT, 0> 
                       LEGO_KNOB_RADIUS-LEGO_KNOB_CORNER_SPACE }
            cylinder { <0, 3*LEGO_HEIGHT-EXTRA, 0> 
                       <0, 3*LEGO_HEIGHT+LEGO_KNOB_HEIGHT-LEGO_KNOB_CORNER_SPACE, 0> 
                       LEGO_KNOB_RADIUS }
            torus { LEGO_KNOB_RADIUS-LEGO_KNOB_CORNER_SPACE  
                    LEGO_KNOB_CORNER_SPACE 
                    translate y*(3*LEGO_HEIGHT+LEGO_KNOB_HEIGHT-LEGO_KNOB_CORNER_SPACE) }
        }

        torus { LEGO_LENGTH/2  
                RR
                translate y*(LEGO_HEIGHT-LEGO_CEILING_HEIGHT)
                }

        

        object { __IAxle  rotate z*90 translate y*(-LEGO_LENGTH+5) }

        cylinder { <0, -1, 0> 
		   <0, LEGO_HEIGHT-LEGO_CEILING_HEIGHT, 0> 
		   LEGO_KNOB_RADIUS }
        cylinder { <0, -1, 0>  
 		   <0, 2*LEGO_LENGTH, 0> 
		   LEGO_KNOB_INNER_RADIUS }

        cylinder { <0, -1, 0> 
		   <0, LEGO_CORNER_SPACE, 0> 
		   LEGO_KNOB_RADIUS+LEGO_CORNER_SPACE }
        cylinder { <0, LEGO_HEIGHT*3+LEGO_KNOB_HEIGHT-LEGO_CORNER_SPACE, 0> 
		   <0, LEGO_HEIGHT*4, 0> 
		   LEGO_KNOB_INNER_RADIUS+LEGO_CORNER_SPACE }
    }
    torus { LEGO_KNOB_RADIUS+LEGO_CORNER_SPACE  
	    LEGO_CORNER_SPACE  
	    translate y*LEGO_CORNER_SPACE }
    torus { LEGO_KNOB_INNER_RADIUS+LEGO_CORNER_SPACE   
	    LEGO_CORNER_SPACE  
            translate y*(LEGO_HEIGHT*3+LEGO_KNOB_HEIGHT-LEGO_CORNER_SPACE) }


    
    torus { LEGO_LENGTH/2 - RR
            LEGO_CORNER_SPACE
            translate y*(LEGO_HEIGHT - LEGO_CEILING_HEIGHT + LEGO_CORNER_SPACE)             }
    
}



#end







