/*******************************************************************************

                      LEGO for POV-Ray 3.5
                     (C) Ernst Martin Witte

   This file is in no way related to the LEGO(tm) Group.
   It is provided for private non-commercial use only.

 *******************************************************************************/

#ifdef (__LEGO_GEAR_INC)
#else

  #declare __LEGO_GEAR_INC = 1;


  #include "roundcylinder.inc"
  #include "roundtube.inc"
  #include "invroundbox.inc"
  #include "inv_round_edge.inc"
  #include "roundbox.inc"
  #include "invroundcylinder.inc"
  #include "roundcone.inc"
  #include "GenericFlat.inc"


  //////////////////////////////////////////////////////////////////////////
  //                           Gear 8
  //////////////////////////////////////////////////////////////////////////
  
  
  #declare Tooth8 = object { 
    roundcylinder(LEGO_LENGTH/2, 1.4/2)
    translate x*LEGO_LENGTH/4
    scale <1, 2, 1>
    translate y * (LEGO_TOOTH_HEIGHT-1.4 + LEGO_BUSH_RADIUS)
  }

  #declare Gear8 = difference {
    union {
      object { roundcylinder(LEGO_LENGTH, LEGO_BUSH_RADIUS) }
	
      #local i = 0;
      #while (i < 8)
	object { Tooth8   rotate x * 360 / 8 * i }
	#declare i = i+1;
      #end
    }
    object { __IAxle translate x*LEGO_LENGTH/2 }
    object { __INV_ROUND_AXLE scale <-1,1,1> }
    object { __INV_ROUND_AXLE translate x*LEGO_LENGTH }

    texture { LegoGray }
  }




  //////////////////////////////////////////////////////////////////////////
  //                           Gear 14
  //////////////////////////////////////////////////////////////////////////


  #declare  Tooth14 = difference {
    intersection {
      box { < 0,  0, -0.75> <1.8, 5, 0.75> }
      box { <-2, -1, -3> <2, 6, 0> rotate y * 10 translate z*0.75}
      box { <-2, -1, 0> <2, 6, 3> rotate -y * 10 translate -z*0.75}
    }
    object {
      inv_roundedge(7,80)
      rotate z*90
      translate <0, -0.5, -0.75>
    }
    object {
      inv_roundedge(7,80)
      rotate z*90
      translate <0, -0.5, -0.75>
      scale <1,1,-1>
    }
    object {
      inv_roundedge(7,100)
      rotate -z*90
      translate <1.8, 6.5, -0.75+1.8*tan(10*pi/180)>
    }
    object {
      inv_roundedge(7,100)
      rotate -z*90
      scale <1, 1, -1>
      translate <1.8, 6.5, 0.75-1.8*tan(10*pi/180)>
    }
    object {
      inv_roundedge(6,90)
      translate -x*0.1
      rotate -y*10
      translate <0, 5, -0.75>
    }
    object {
      inv_roundedge(6,90)
      scale <1, 1, -1>
      translate -x*0.1
      rotate y*10
      translate <0, 5, 0.75>
    }
    object {
      inv_roundedge(3,90)
      rotate y*90
      translate <0, 5, 1.5>
    }
    box {
        <0, 1, -1><3, -5, 1>
        rotate z*45
        translate <0.6, 5, 0>
    }
    object {
      inv_roundedge(3,135)
      rotate y*90
      scale <-1, 1, 1>
      translate <0.6, 5, 1.5>
    }
    object {
      inv_roundedge(3,135)
      rotate y*90
      rotate -z*90
      translate <1.8, 5-1.2, 1.5>
    }
    #local __TEMP_a = <1.8, 5-1.2, -0.75+1.8*tan(10*pi/180) > - <0.6, 5, -0.75+0.6*tan(10*pi/180) >;
    #local __TEMP_b = -z;
    #local __TEMP_c = -y;
    #local __TEMP_x_ =  vcross(__TEMP_a, vcross (__TEMP_a, __TEMP_b));
    #local __TEMP_y_ = -vcross(__TEMP_a, vcross (__TEMP_a, __TEMP_c));
    #local __TEMP_phi1 = acos (vdot (__TEMP_x_, __TEMP_y_) / vlength(__TEMP_x_) / vlength(__TEMP_y_)) * 180/pi;
    #local __TEMP_u =vrotate (x, -z*45);
    #local __TEMP_phi2 = (acos(vdot (__TEMP_a, __TEMP_u) / vlength(__TEMP_a) / vlength(__TEMP_u))*180/pi);
    object {
      inv_roundedge(3,__TEMP_phi1)
      translate -x*2.9
      rotate -y*__TEMP_phi2
      rotate -z*45
      translate <1.8, 5-1.2, -0.75+1.8*tan(10*pi/180) >
    }
    object {
      inv_roundedge(3,__TEMP_phi1)
      scale <1, 1, -1>
      translate -x*2.9
      rotate y*__TEMP_phi2
      rotate -z*45
      translate <1.8, 5-1.2, 0.75-1.8*tan(10*pi/180) >
    }

    translate x*1.0 + y*2.4
  }
  


  #declare Gear14 = difference {
    union {
      object {
	RoundCone (5.6, 4, 1.8)
	rotate -z*90
	translate x*1
      }
      object {
	h_roundtube(2, 4.6, 3.4)
      }
      #declare i = 0;
      #while (i < 14)
	object { Tooth14 rotate x*360/14*i }
	#declare i = i + 1;
      #end
    }
    object { __IAxle  }
    object { __INV_ROUND_AXLE translate x*2.8 }
    object { __INV_ROUND_AXLE rotate y*180 translate x*1 }
    texture { LegoGray }
  }

  
  
  
  //////////////////////////////////////////////////////////////////////////
  //                           Gear 16
  //////////////////////////////////////////////////////////////////////////

  #declare Tooth16 = difference {
    intersection {
      box { <-0.1, -0.5, -1> <LEGO_LENGTH/2 + 0.1, LEGO_TOOTH_HEIGHT, 1>}
      object {
	roundcylinder(LEGO_LENGTH/2, LEGO_WALL_WIDTH/2)
	scale <1, 2.2, 1>
	translate y*0.4
      }
    }
    object {
        INV_CORNER_CYLINDER
        scale <1, 2.2, 1>
        rotate -y*90
        translate <0, LEGO_TOOTH_HEIGHT, -LEGO_LENGTH>
    }
    object {
        INV_CORNER_CYLINDER
        scale <1, 2.2, -1>
        rotate -y*90
        translate <LEGO_LENGTH/2, LEGO_TOOTH_HEIGHT, -LEGO_LENGTH>
    }

    translate x * LEGO_LENGTH/4
  }


  #declare Gear16 = difference {
    union {
      object {
	roundtube(LEGO_LENGTH/2, LEGO_GEAR16_RADIUS, LEGO_GEAR16_RADIUS - 1)
	translate x*LEGO_LENGTH/4
      }
      cylinder { <LEGO_LENGTH/4+1, 0, 0> <LEGO_LENGTH*3/4-1, 0, 0> LEGO_GEAR16_RADIUS - EXTRA }

      #local i = 0;
      #while (i < 16)
	object { Tooth16 translate y*LEGO_GEAR16_RADIUS - EXTRA  rotate x * 360 / 16 * i }
	#local i = i+1;
      #end

      #local a = 0;
      #while (a < 4)
	object {
	  roundtube (LEGO_LENGTH/2, 2.8, LEGO_KNOB_INNER_RADIUS)
	  translate <LEGO_LENGTH/4, LEGO_GEAR16_RADIUS - 2.8 - EXTRA, 0>
	  rotate x*a*90
	}
	#local a = a + 1;
      #end
      object { roundcylinder (LEGO_LENGTH, LEGO_TUBE_RADIUS) }
    }
    #local a = 0;
    #while (a < 4)
      object {
	invroundcylinder (LEGO_LENGTH, LEGO_KNOB_INNER_RADIUS+EXTRA)
	translate y*(LEGO_GEAR16_RADIUS - 2.8 - EXTRA)
	rotate x*a*90
      }
      #local a = a + 1;
    #end

    union {
      object { 
	invroundbox(LEGO_LENGTH, LEGO_KNOB_RADIUS*3+2*EXTRA, LEGO_CROSSAXLE_WIDTH+2*EXTRA)
	translate <0, -LEGO_KNOB_RADIUS*1.5-EXTRA, -LEGO_CROSSAXLE_WIDTH/2-EXTRA>
      }
      object { 
	invroundbox(LEGO_LENGTH, LEGO_CROSSAXLE_WIDTH+2*EXTRA, LEGO_KNOB_RADIUS*3+2*EXTRA)
	translate <0, -LEGO_CROSSAXLE_WIDTH/2-EXTRA, -LEGO_KNOB_RADIUS*1.5-EXTRA>
      }
    }
    object { __IAxle translate x*0.5*l}
    object { __INV_ROUND_AXLE scale <-1,1,1> }
    object { __INV_ROUND_AXLE translate x*LEGO_LENGTH }
    
    texture { LegoGray }
  }
  
  
  
  //////////////////////////////////////////////////////////////////////////
  //                           GenericTooth
  //////////////////////////////////////////////////////////////////////////
  
  
  #declare ToothXX = difference {
    
    #local __TEMP_fb = (LEGO_TOOTH_HEIGHT+0.15+0.3) / (LEGO_WALL_WIDTH/2);
    #local __TEMP_fa = 1.2;

    intersection {
      box { <-0.1, -0.5, -1> <LEGO_LENGTH/2 + 0.1, LEGO_TOOTH_HEIGHT, 1>}
      object {
	roundcylinder(LEGO_LENGTH/2, LEGO_WALL_WIDTH/2)
	scale <1, __TEMP_fb, __TEMP_fa>
	translate -y*0.3
      }
    }
    object {
      INV_CORNER_CYLINDER
      scale <1.2, 2.7, 1>
      rotate -y*90
      translate <0, LEGO_TOOTH_HEIGHT, -LEGO_LENGTH>
    }
    object {
      INV_CORNER_CYLINDER
      scale <1.2, 2.7, -1>
      rotate -y*90
      translate <LEGO_LENGTH/2, LEGO_TOOTH_HEIGHT, -LEGO_LENGTH>
    }

    #local __TEMP_a_g = LEGO_WALL_WIDTH/2 * __TEMP_fa;
    #local __TEMP_b_g = LEGO_WALL_WIDTH/2 * __TEMP_fb;
    #local __TEMP_y_g = LEGO_TOOTH_HEIGHT + 0.3;
    #local __TEMP_x_g = __TEMP_a_g * sqrt (1 - pow (__TEMP_y_g/__TEMP_b_g, 2));
            
    #local Angle  = 180 + atan2 (-__TEMP_x_g * pow(__TEMP_b_g, 2), __TEMP_y_g*pow(__TEMP_a_g, 2)) * 180 / pi;
            
    object {
      inv_roundedge (LEGO_LENGTH, Angle)
      translate <-LEGO_LENGTH/4, LEGO_TOOTH_HEIGHT, -__TEMP_x_g>
    }        
    object {
      inv_roundedge (LEGO_LENGTH, Angle)
      scale <1, 1, -1>
      translate <-LEGO_LENGTH/4, LEGO_TOOTH_HEIGHT, __TEMP_x_g>
    }           

    translate x * LEGO_LENGTH/4
  }


  
  //////////////////////////////////////////////////////////////////////////
  //                           Gear 24
  //////////////////////////////////////////////////////////////////////////


  #declare Tooth24 = object {
    ToothXX
    translate y * LEGO_GEAR24_RADIUS
  }


  #declare Gear24 = difference {
    union {
      difference {
	union {
	  #local i = 0;
	  #while (i < 24)
	    object { Tooth24 rotate x*360/24*i }
	    #local i=i+1;
	  #end
	  object { roundtube(LEGO_LENGTH/2, LEGO_GEAR24_RADIUS, LEGO_GEAR24_RADIUS - 1)  translate x*LEGO_LENGTH/4 }
	  cylinder { <LEGO_LENGTH/4 + 1.2, 0, 0> <LEGO_LENGTH*3/4 - 1.2, 0, 0> LEGO_GEAR24_RADIUS - 1 + EXTRA }
	  
	  
	  object { roundbox(LEGO_LENGTH, 10, 10) translate <0, -5, -5> }
	}
	
	#local i = 0;
	#while (i < 4)
	  union {
	    cylinder { (-x+y+z)*LEGO_LENGTH/2   (2.1*x+y+z)*LEGO_LENGTH/2     LEGO_KNOB_RADIUS+LEGO_WALL_WIDTH/4  }
	    object { 
	      h_invroundcylinder(LEGO_WALL_WIDTH/2, LEGO_KNOB_RADIUS+LEGO_WALL_WIDTH/2)
	      rotate y*180  translate (2*x+y+z)*LEGO_LENGTH/2 }
	    object { 
	      h_invroundcylinder(LEGO_WALL_WIDTH/2, LEGO_KNOB_RADIUS+LEGO_WALL_WIDTH/2)
	      translate (    y+z)*LEGO_LENGTH/2 }
	    
	    rotate x*90*i
	  }
	  #local i=i+1;
	#end
	
      }
      #local i = 0;
      #while (i < 4)
	object { roundtube(LEGO_LENGTH - LEGO_WALL_WIDTH, LEGO_KNOB_RADIUS + LEGO_WALL_WIDTH/2, LEGO_KNOB_RADIUS)
	  translate (y+z)*LEGO_LENGTH/2+x*LEGO_WALL_WIDTH/2 rotate x*90*i }
	#local i=i+1;
      #end
    }
    
    
    
    object { invroundbox(LEGO_LENGTH-LEGO_WALL_WIDTH, LEGO_CROSSAXLE_WIDTH/sqrt(2)+0.2, 6)
      translate <LEGO_WALL_WIDTH/2,  LEGO_LENGTH/2-0.2, -3> }

    object { invroundbox(LEGO_LENGTH-LEGO_WALL_WIDTH, LEGO_CROSSAXLE_WIDTH/sqrt(2)+0.2, 6)
      translate <LEGO_WALL_WIDTH/2,  LEGO_LENGTH/2-0.2, -3> scale <1,-1,1>}

    object { __IAxle translate x*EXTRA}
    object { __INV_ROUND_AXLE translate x*LEGO_LENGTH }
    object { __INV_ROUND_AXLE rotate y*180 }
    
    object { invroundbox(LEGO_LENGTH, 3/2*LEGO_LENGTH, LEGO_CROSSAXLE_WIDTH-2*EXTRA)
      translate <0, -1.5/2*LEGO_LENGTH, -LEGO_CROSSAXLE_WIDTH/2+EXTRA> }
    
    intersection {
      object { invroundbox(LEGO_LENGTH-LEGO_WALL_WIDTH, 3/2*LEGO_LENGTH, LEGO_CROSSAXLE_WIDTH+2*EXTRA)
	translate <LEGO_WALL_WIDTH/2, -1.5/2*LEGO_LENGTH, -LEGO_CROSSAXLE_WIDTH/2-EXTRA> }
      union {
	cylinder { ( y+z)*LEGO_LENGTH/2, ( y+z)*LEGO_LENGTH/2+x*LEGO_LENGTH, LEGO_KNOB_RADIUS + LEGO_WALL_WIDTH/2 }
	cylinder { ( y-z)*LEGO_LENGTH/2, ( y-z)*LEGO_LENGTH/2+x*LEGO_LENGTH, LEGO_KNOB_RADIUS + LEGO_WALL_WIDTH/2 }
	cylinder { (-y+z)*LEGO_LENGTH/2, (-y+z)*LEGO_LENGTH/2+x*LEGO_LENGTH, LEGO_KNOB_RADIUS + LEGO_WALL_WIDTH/2 }
	cylinder { (-y-z)*LEGO_LENGTH/2, (-y-z)*LEGO_LENGTH/2+x*LEGO_LENGTH, LEGO_KNOB_RADIUS + LEGO_WALL_WIDTH/2 }
      }
    }

    #local i = 0;
    #while ( i < 2)
      union {
	object { __IAxle rotate x*45  }
	object { __INV_ROUND_AXLE translate x*(LEGO_LENGTH-LEGO_WALL_WIDTH/2) rotate x*45 }
	object { __INV_ROUND_AXLE rotate y*180 translate x*LEGO_WALL_WIDTH/2  rotate x*45 }
	translate y*1.5/2*LEGO_LENGTH 
	rotate x*i*180
      }
      difference {
	union {
	  object { __INV_ROUND_AXLE translate x*(LEGO_LENGTH*3/4-1.2) rotate x*45 }
	  object { __INV_ROUND_AXLE rotate y*180 translate x*(LEGO_LENGTH/4+1.2) rotate x*45 }
	  
	  object { invroundbox(LEGO_LENGTH/2-2.4, LEGO_KNOB_RADIUS+1, 0.8) translate <LEGO_LENGTH/4+1.2, 0, -0.4> rotate  x*45 }
	  object { invroundbox(LEGO_LENGTH/2-2.4, LEGO_KNOB_RADIUS+1, 0.8) translate <LEGO_LENGTH/4+1.2, 0, -0.4> rotate -x*45 }
	  
	  object { invroundcylinder(LEGO_LENGTH/2-2.4, 0.4) translate <LEGO_LENGTH/4+1.2, LEGO_KNOB_RADIUS+1, 0> rotate  x*45 }
	  object { invroundcylinder(LEGO_LENGTH/2-2.4, 0.4) translate <LEGO_LENGTH/4+1.2, LEGO_KNOB_RADIUS+1, 0> rotate -x*45 }
	  
	  translate y*1.5/2*LEGO_LENGTH
	}
	
	cylinder { <-1, LEGO_LENGTH/2,  LEGO_LENGTH/2> <LEGO_LENGTH, LEGO_LENGTH/2,  LEGO_LENGTH/2> LEGO_KNOB_RADIUS + LEGO_WALL_WIDTH/2 + EXTRA} 
	cylinder { <-1, LEGO_LENGTH/2, -LEGO_LENGTH/2> <LEGO_LENGTH, LEGO_LENGTH/2, -LEGO_LENGTH/2> LEGO_KNOB_RADIUS + LEGO_WALL_WIDTH/2 + EXTRA} 

	rotate x*i*180
      }	
      #local i = i + 1;
    #end
    
    texture { LegoGray }
  }






  //////////////////////////////////////////////////////////////////////////
  //                           Gear 24e
  //////////////////////////////////////////////////////////////////////////



  #local  __TEMP_TOOTH_WIDTH = 1.4;
  #local  __TEMP_TOOTH_R     = 2;

  #declare Tooth24e = union {
    intersection {
      object {
	roundcylinder(__TEMP_TOOTH_WIDTH, __TEMP_TOOTH_R * 2 / 3)
	rotate y*90
	scale <3/2, 1, 1>
	translate <2.8, LEGO_GEAR24_RADIUS, __TEMP_TOOTH_WIDTH / 2>
      }
      box { <0, LEGO_GEAR24_RADIUS + 2*__TEMP_TOOTH_R, -__TEMP_TOOTH_WIDTH> <LEGO_LENGTH, LEGO_GEAR24_RADIUS - 0.5,  __TEMP_TOOTH_WIDTH> }
    }
    intersection {
      object {
	roundbox((__TEMP_TOOTH_R+1)*2/3, 2, __TEMP_TOOTH_WIDTH)
	scale <3/2, 1, 1>
	translate <1.8, LEGO_GEAR24_RADIUS - 1.5, -__TEMP_TOOTH_WIDTH/2>
      }
      box { <0.8, LEGO_GEAR24_RADIUS, -__TEMP_TOOTH_WIDTH> <LEGO_LENGTH, LEGO_GEAR24_RADIUS - 3, __TEMP_TOOTH_WIDTH> }
    }
    bounded_by { box { <0.7, LEGO_GEAR24_RADIUS - 3, -__TEMP_TOOTH_WIDTH> <LEGO_LENGTH, LEGO_GEAR24_RADIUS + 3, __TEMP_TOOTH_WIDTH> } }
  }
  
  
  #declare Gear24e = difference {
    #local __TEMP_INNER_RADIUS = 6.3;
    union {
      object {
	roundtube(2, LEGO_GEAR24_RADIUS, __TEMP_INNER_RADIUS)
	translate x*0.8
      }
      torus {
	#local __TEMP_x = (LEGO_GEAR24_RADIUS - __TEMP_INNER_RADIUS) / 2 / 3;
	#local ____r = (__TEMP_x * __TEMP_x + 0.4*0.4) / (2*0.4);
	(LEGO_GEAR24_RADIUS + __TEMP_INNER_RADIUS) / 2,  ____r
	rotate z*90
	translate x*(____r + 0.8 - 0.4)
      }
      object {
	roundcylinder(LEGO_LENGTH, 3.7)
      }
      object { roundbox(2, 5, 14) translate <0.8, -2.5, -7> }
      #local i=0;
      #while (i<24)
	object { Tooth24e rotate x*360/24*i }
	#local i = i+1;
      #end
    }
    object { __IAxle translate x*LEGO_LENGTH/2}
    object { __INV_ROUND_AXLE translate x*LEGO_LENGTH}
    object { __INV_ROUND_AXLE rotate y*180}

    object { invroundbox(LEGO_LENGTH, LEGO_GAP_WIDTH, 10.6)
      translate <0,-LEGO_GAP_WIDTH/2, -5.3> }
    difference {
      object { invroundbox(2, LEGO_GAP_WIDTH+EXTRA*2, 10.6)
	translate <0.8,-LEGO_GAP_WIDTH/2-EXTRA, -5.3> }
      cylinder { <-1, 0, 0> <LEGO_LENGTH+1,0,0> 3.7 }
    }
    object { invroundcylinder(2, LEGO_GAP_WIDTH/2) translate <0.8, 0, -5.3> }
    object { invroundcylinder(2, LEGO_GAP_WIDTH/2) translate <0.8, 0,  5.3> }

    texture { LegoGray }
  }

  
  
  //////////////////////////////////////////////////////////////////////////
  //                           Gear 40
  //////////////////////////////////////////////////////////////////////////

  #declare Tooth40 = object {
    ToothXX
    translate y * LEGO_GEAR40_RADIUS
  }

  #declare Gear40 = union {
    difference {
      union {
	#local i = 0;
	#while (i < 40)
	  object { Tooth40     rotate x*360/40*i }
	  #local i=i+1;
	#end
	object { roundtube(LEGO_LENGTH/2, LEGO_GEAR40_RADIUS, LEGO_GEAR40_RADIUS-1.3) translate x*LEGO_LENGTH/4 }
	cylinder { <LEGO_LENGTH/4 + 1.2, 0, 0> <LEGO_LENGTH*3/4 - 1.2, 0, 0> LEGO_GEAR40_RADIUS - 1.3 + EXTRA }
	
	
	object { roundbox( LEGO_LENGTH/2, 1.5, LEGO_GEAR40_RADIUS*2-1) translate <LEGO_LENGTH/4, -0.75, -LEGO_GEAR40_RADIUS+0.5> }
	object { roundbox( LEGO_LENGTH/2, LEGO_GEAR40_RADIUS*2-1, 1.5) translate <LEGO_LENGTH/4, -LEGO_GEAR40_RADIUS+0.5, -0.75> }

	intersection {
	  object { roundbox (LEGO_LENGTH-LEGO_WALL_WIDTH, LEGO_LENGTH*3, LEGO_LENGTH*3)
	    translate <LEGO_WALL_WIDTH/2, -LEGO_LENGTH*1.5, -LEGO_LENGTH*1.5> }
	  object { roundbox (LEGO_LENGTH-LEGO_WALL_WIDTH+2*EXTRA, LEGO_LENGTH*3, LEGO_LENGTH*3)
	    translate <LEGO_WALL_WIDTH/2-EXTRA, -LEGO_LENGTH*1.5, -LEGO_LENGTH*1.5> 
	    rotate x*45 }
	}



	#local i = 0;
	#while (i < 2)
	  difference {
	    #local b = LEGO_WALL_WIDTH/2;
	    #local a = LEGO_LENGTH/2;
	    prism {
	      conic_sweep
	      linear_spline
	      1-b/a
	      1
	      5
	      <-a-b, -a> <a+b, -a> <a+b, a> <-a-b, a> <-a-b, -a>
	      translate -y
	      scale <1, -a, 1>
	    }
	    #local j=0;
	    #while (j<2)
	      union {
		object { INV_CORNER_CYLINDER135  translate <-LEGO_LENGTH, b, a-b> }
		object { INV_CORNER_CYLINDER135  translate <-LEGO_LENGTH, b, a-b*b/a> rotate y*90}
		rotate y*j*180
	      } 
	      #local j=j+1;
	    #end
	    
	    rotate -z*90*(1-2*i)
	    translate <LEGO_LENGTH/2 + (1-2*i)*(LEGO_LENGTH/2 - b), 0, 0>
	    
	  }
	  #local i = i+1;
	#end
	
      }

      #local i = 0;
      #while (i < 4)
	union {
	  cylinder { <-1, LEGO_LENGTH/2,   -LEGO_LENGTH/2> <LEGO_LENGTH+1, LEGO_LENGTH/2,   -LEGO_LENGTH/2>, LEGO_KNOB_RADIUS+LEGO_WALL_WIDTH/4 }
	  cylinder { <-1, LEGO_LENGTH*3/2, -LEGO_LENGTH/2> <LEGO_LENGTH+1, LEGO_LENGTH*3/2, -LEGO_LENGTH/2>, LEGO_KNOB_RADIUS+LEGO_WALL_WIDTH/4 }
	  cylinder { <-1, LEGO_LENGTH*3/2, +LEGO_LENGTH/2> <LEGO_LENGTH+1, LEGO_LENGTH*3/2, +LEGO_LENGTH/2>, LEGO_KNOB_RADIUS+LEGO_WALL_WIDTH/4 }

	  object { h_invroundcylinder (LEGO_WALL_WIDTH/2 + EXTRA, LEGO_KNOB_RADIUS+LEGO_WALL_WIDTH/2)
	    translate (y+z)*LEGO_LENGTH/2 }
	  object { h_invroundcylinder (LEGO_WALL_WIDTH/2 + EXTRA, LEGO_KNOB_RADIUS+LEGO_WALL_WIDTH/2)
	    rotate y*180  translate x*LEGO_LENGTH+(y+z)*LEGO_LENGTH/2 }

	  object { __IAxle  translate <0, LEGO_LENGTH, 0> }
	  object { __INV_ROUND_AXLE  translate <LEGO_LENGTH-LEGO_WALL_WIDTH/2, LEGO_LENGTH, 0> }
	  object { __INV_ROUND_AXLE  rotate y*180 translate <0, LEGO_LENGTH, 0> }
	  
	  object { invroundbox(LEGO_LENGTH-LEGO_WALL_WIDTH,0.8,12) translate <LEGO_WALL_WIDTH/2, LEGO_LENGTH-0.4,-6> }
	  object { invroundcylinder(LEGO_LENGTH-LEGO_WALL_WIDTH,0.4) translate <LEGO_WALL_WIDTH/2, LEGO_LENGTH, -6> }
	  object { invroundcylinder(LEGO_LENGTH-LEGO_WALL_WIDTH,0.4) translate <LEGO_WALL_WIDTH/2, LEGO_LENGTH, +6> }
	    
	  rotate x*i*90
	}
	#local i = i+1;
      #end
      
      object { __IAxle translate x*LEGO_LENGTH/2 }
      object { __INV_ROUND_AXLE translate x*LEGO_LENGTH }
      object { __INV_ROUND_AXLE rotate y*180 }
      object { invroundbox(LEGO_LENGTH, 0.8, LEGO_LENGTH+0.8) translate <0, -0.4, -0.4-LEGO_LENGTH/2> }
      difference {
	union {
	  object { invroundcylinder (LEGO_LENGTH-LEGO_WALL_WIDTH, 0.4) translate <LEGO_WALL_WIDTH/2, 0, -LEGO_LENGTH/2-0.4> }
	  object { invroundcylinder (LEGO_LENGTH-LEGO_WALL_WIDTH, 0.4) translate <LEGO_WALL_WIDTH/2, 0,  LEGO_LENGTH/2+0.4> }
	}
	box { -LEGO_LENGTH/2+EXTRA  LEGO_LENGTH/2-EXTRA }
      }
    }
    
    #local i = 0;
    #while (i < 4)
      union {
	object { roundtube(LEGO_LENGTH - LEGO_WALL_WIDTH, LEGO_KNOB_RADIUS+LEGO_WALL_WIDTH/2, LEGO_KNOB_RADIUS) translate (y+z)*LEGO_LENGTH/2 }
	object { roundtube(LEGO_LENGTH - LEGO_WALL_WIDTH, LEGO_KNOB_RADIUS+LEGO_WALL_WIDTH/2, LEGO_KNOB_RADIUS) translate (y*3+z)*LEGO_LENGTH/2 }
	object { roundtube(LEGO_LENGTH - LEGO_WALL_WIDTH, LEGO_KNOB_RADIUS+LEGO_WALL_WIDTH/2, LEGO_KNOB_RADIUS) translate (y*3-z)*LEGO_LENGTH/2 }
	
	translate x*LEGO_WALL_WIDTH/2
	rotate x*90*i
      }
      #local i = i+1;
    #end

    texture { LegoGray }
}





  //////////////////////////////////////////////////////////////////////////
  //                           Differential Gear 
  //////////////////////////////////////////////////////////////////////////


  
  #declare Tooth28 = object {
    ToothXX
    translate -x*LEGO_LENGTH/2
    scale <3.3 / (LEGO_LENGTH/2), 1, 1>
    translate <LEGO_LENGTH-2.5 + 3.3/2, LEGO_GEAR28_RADIUS, 0>
  }


  #declare DifferentialGear = union {
    difference {
      union {
	object { roundcylinder(LEGO_LENGTH-1, LEGO_LENGTH/2) }
	object { roundcylinder(LEGO_LENGTH-1, LEGO_LENGTH/2) rotate y*180 translate x*4*LEGO_LENGTH}
	object { roundtube (3.3, LEGO_GEAR28_RADIUS, LEGO_GEAR28_RADIUS - 2.4) translate x*(LEGO_LENGTH-2.5) }

	cylinder { <LEGO_LENGTH - 2.5, 0, 0> <LEGO_LENGTH, 0, 0> LEGO_GEAR28_RADIUS - 1.2 }
      
	difference {
	  intersection {
	    object { roundcylinder (3*LEGO_LENGTH+2.5+EXTRA, LEGO_GEAR28_RADIUS - 2.4) } 
	    object { roundbox (2*LEGO_LENGTH+2.5, LEGO_LENGTH, 4*LEGO_LENGTH)
	      translate <LEGO_LENGTH, -LEGO_LENGTH/2, -2*LEGO_LENGTH> }
	  }
	  object { invroundbox(LEGO_LENGTH, 2*LEGO_LENGTH, 2*LEGO_LENGTH) 
	    rotate -z*90
	    translate <LEGO_LENGTH, LEGO_LENGTH/2, -LEGO_LENGTH>  }
	}
      }
      
      object { invroundcylinder(LEGO_LENGTH, LEGO_KNOB_RADIUS) }
      object { invroundcylinder(LEGO_LENGTH, LEGO_KNOB_RADIUS) rotate y*180 translate x*4*l }
    }
    object {
      roundcylinder(LEGO_LENGTH + 1 - (LEGO_GEAR28_RADIUS - 2.4 - 6), LEGO_CROSSAXLE_WIDTH/sqrt(2))
      rotate y*90
      translate <2*LEGO_LENGTH, 0, LEGO_LENGTH+1>
    }
    
    #local i=0;
    #while (i < 28)
      object { Tooth28  rotate x*360/28*i }
      #local i=i+1;
    #end

    texture { LegoGray }
  } 

  
  //////////////////////////////////////////////////////////////////////////
  //                           Linear teeth
  //////////////////////////////////////////////////////////////////////////


  #declare ___TEMP_LINEAR_GEAR_dh = 0.3;
  #declare ___TEMP_LINEAR_GEAR_a = 0.8;
  #declare ___TEMP_LINEAR_GEAR_b = 4*LEGO_LENGTH/10 - ___TEMP_LINEAR_GEAR_a;
  #declare ___TEMP_LINEAR_GEAR_h = 5.9-h-___TEMP_LINEAR_GEAR_dh;
  #declare ___TEMP_LINEAR_GEAR_phi = 90 - atan2(___TEMP_LINEAR_GEAR_h, (___TEMP_LINEAR_GEAR_b-___TEMP_LINEAR_GEAR_a)/2)*180/pi;


  #declare LTooth = difference {
    intersection {
      box {
	<0, -LEGO_CORNER_SPACE, 0>
	<___TEMP_LINEAR_GEAR_b, ___TEMP_LINEAR_GEAR_h, LEGO_LENGTH>
      }
      box {
	<0, -2*LEGO_CORNER_SPACE, -LEGO_LENGTH>
	<___TEMP_LINEAR_GEAR_b, 2*___TEMP_LINEAR_GEAR_h, LEGO_LENGTH*2>
	rotate -z*___TEMP_LINEAR_GEAR_phi
      }
      box {
	<-___TEMP_LINEAR_GEAR_b, -2*LEGO_CORNER_SPACE, -LEGO_LENGTH>
	<0, 2*___TEMP_LINEAR_GEAR_h, LEGO_LENGTH*2>
	rotate z*___TEMP_LINEAR_GEAR_phi
	translate x*___TEMP_LINEAR_GEAR_b
      }
    }
    
    #local __re = object { inv_roundedge(LEGO_LENGTH, 90)
      translate -x*2*LEGO_CORNER_SPACE
      rotate z*(90-___TEMP_LINEAR_GEAR_phi)
    }
    
    #local __re2 = object { inv_roundedge(LEGO_LENGTH, 90+___TEMP_LINEAR_GEAR_phi)
      rotate -y*90
      scale <-1,1,1>
      translate y*___TEMP_LINEAR_GEAR_h+x*___TEMP_LINEAR_GEAR_h*tan(___TEMP_LINEAR_GEAR_phi/180*pi)
    }
    
    object { __re2 }
    object { __re2 scale <-1,1,1> translate x*___TEMP_LINEAR_GEAR_b }
    object { __re }
    object { __re scale <-1,1,1>  translate x*___TEMP_LINEAR_GEAR_b }
    object { __re scale <1,1,-1>  translate z*LEGO_LENGTH }
    object { __re scale <-1,1,-1> translate x*___TEMP_LINEAR_GEAR_b +z*LEGO_LENGTH }
    object { inv_roundedge(LEGO_LENGTH, 90) translate y*___TEMP_LINEAR_GEAR_h }
    object { inv_roundedge(LEGO_LENGTH, 90) scale <1,1,-1> translate y*___TEMP_LINEAR_GEAR_h+x*LEGO_LENGTH }
      
    
    translate  <___TEMP_LINEAR_GEAR_a/2, h+___TEMP_LINEAR_GEAR_dh, 0>
  }


  #macro LinearGear(Length) 
  
  
  union {
    object {
      GenericFlat(Length,1)
    }
    object {
      h_roundbox(Length*LEGO_LENGTH, ___TEMP_LINEAR_GEAR_dh+LEGO_CORNER_SPACE, LEGO_LENGTH)
      translate y*(h-LEGO_CORNER_SPACE)
    }
    #local i = 0;
    #while (i < Length/4 * 10)
      object {
	LTooth
	translate x*i*4*LEGO_LENGTH/10
      }
      #local i = i+1;
    #end
    texture { LegoGray }
  }
#end
  
  
#end
