/*******************************************************************************

                      LEGO for POV-Ray 3.5
                     (C) Ernst Martin Witte

   This file is in no way related to the LEGO(tm) Group.
   It is provided for private non-commercial use only.

   inv_round_edge.inc: Inverses zum Erzeugen von runden Kanten an beliebigen Winkeln


 *******************************************************************************/

#ifdef (__INV_ROUND_CONE_EDGE_INC)
#else
  
  #declare __INV_ROUND_CONE_EDGE_INC = 1;

  
  #macro invround_coneedge(Radius, ConeAngle, RotAngle)
    
    invround_coneedgeX(Radius, ConeAngle, RotAngle, LEGO_CORNER_SPACE)
    
  #end
  
  // concave version: add this macro (union/merge)
  // convex  version: sub this macro (difference)

  // Radius:    sharp edge will _always_ be on x-z-plane, radius "Radius"
  // ConeAngle: opening angle
  //
  //  y                     /      Angle ~= 135 in this example
  //  ^                    /
  //  |->x       _________/
  // 
  //  RotAngle: "rotate" around edge, leftwise in the example above
  
  #macro invround_coneedgeX(Radius, ConeAngle, RotAngle, CRadius)
  
    #local __RR = CRadius * 1;
    
    #if((ConeAngle <= 0) | (ConeAngle >= 180))
      #error concat ("invround_coneedge: ConeAngle out of range (", str(ConeAngle,4,4), ")")
    #end
    
    #local __PHI     = 180-ConeAngle;
    #local __TEMP_R_ = __RR/cos(__PHI/2*pi/180);
    #local __DX      = __TEMP_R_ * sin(__PHI/2*pi/180);
    #local __P       = <-__DX, __RR, 0>;
    #local __P       = vrotate(__P, z*RotAngle);
    #local __P_      = <-__DX+__RR*cos((90-__PHI)*pi/180), -__RR*sin((90-__PHI)*pi/180), 0>;
    #local __P_      = vrotate(__P_, z*RotAngle);
    
    //  #warning concat ("phi = ",  str(__PHI,4,4))
    //  #warning concat ("DX = ",   str(__DX,4,4))
    //  #warning concat ("p.x = ",  str(__P.x,4,4))
    //  #warning concat ("p'.x = ", str(__P_.x,4,4))
    
    difference {
      union {
	torus {
	  Radius + __P_.x
	  __RR
	  translate y*__P_.y
	}
	torus { Radius, __DX }
      }
      torus {
	Radius + __P.x
	__RR
	translate y*__P.y
      }
    }
  
  #end 
  
#end
