/*******************************************************************************

                      LEGO for POV-Ray 3.5
                     (C) Ernst Martin Witte

   This file is in no way related to the LEGO(tm) Group.
   It is provided for private non-commercial use only.

   LEGOBASE.INC: constants and textures for LEGO for POV-Ray

   Objects:     LEGO_LOGO :     text "LEGO" on most knobs
                KNOB :          knob, used on most LEGO bricks
                INV_INNER_KNOB: little hole unter the knobs
                SCYLINDER1 :    small cylinder, used under bricks with width 1 and height 1
                SCYLINDER3 :    small cylinder, used under bricks with width 1 and height 3
                CYLINDER1 :     cylinder, used under bricks with width >= 2 and height 1
                CYLINDER3 :     cylinder, used under bricks with width >= 2 and height 3

 *******************************************************************************/

#ifdef (__LEGO_BASE_INC)
#else

    #declare __LEGO_BASE_INC = 1;
  
  


  #include "lego_colors.inc"
  #include "roundcylinder.inc"
  #include "invroundcylinder.inc"
  #include "roundtube.inc"



// All constants are measured in mm

#declare LEGO_LENGTH                = 8;                                    // basic Length for brickwidth and -length
#declare LEGO_HEIGHT                = LEGO_LENGTH*2/5;                      // basic height for bricks = height of plates
#declare LEGO_WALL_WIDTH            = LEGO_HEIGHT / 2;
#declare LEGO_CEILING_HEIGHT        = 1.2;

#declare LEGO_KNOB_HEIGHT           = LEGO_HEIGHT / 2;
#declare LEGO_KNOB_RADIUS           = (LEGO_LENGTH - LEGO_HEIGHT) / 2;
#declare LEGO_KNOB_INNER_RADIUS     = LEGO_HEIGHT / 2;                      // Radius in Technic-Knobs

#declare LEGO_INV_KNOB_HEIGHT       = 2.0;                                  // non-technic: the hole under the knob
#declare LEGO_INV_KNOB_RADIUS       = 2.5/2;

#declare LEGO_SCYLINDER_RADIUS      = LEGO_HEIGHT / 2;
#declare LEGO_SCYLINDER_INNER_RADIUS= 1.2 / 2;

#declare LEGO_CYLINDER_RADIUS       = LEGO_LENGTH/sqrt(2) - LEGO_KNOB_RADIUS;
#declare LEGO_CYLINDER_WALL_WIDTH   = LEGO_CYLINDER_RADIUS - LEGO_KNOB_RADIUS;

#declare LEGO_CORNER_SPACE          = 0.20;
#declare LEGO_KNOB_CORNER_SPACE     = 0.375;


#declare EXTRA                      = 0.0003;                              // avoid ugly effects in unions and merges with identical surfaces



// Abbreviations, names are grown, please do not interprete ;)
#declare l                          = LEGO_LENGTH;
#declare h                          = LEGO_HEIGHT;



  // Lego-Logo

  #declare LEGO_LOGO = text {
    ttf "ariali.ttf"
    "LEGO"
    LEGO_KNOB_CORNER_SPACE/2
    0
    matrix <
    1, 0, 0,
    0.4, 1, 0,
    0, 0, 1,
    0, 0, 0
    >
    translate <0, -0.33, 0>
    scale <1.1, LEGO_KNOB_RADIUS*1.3, 1>
    rotate x * 90
    translate <-LEGO_KNOB_RADIUS * 0.67, LEGO_KNOB_CORNER_SPACE/2, 0>
    rotate y * -90
  }


  // Knobs

  #declare KNOB = union {
    object {
      h_roundcylinderX(LEGO_KNOB_HEIGHT+LEGO_CORNER_SPACE, LEGO_KNOB_RADIUS, LEGO_KNOB_CORNER_SPACE)
      rotate -z*90
      translate <0, LEGO_KNOB_HEIGHT, 0>
    }
    object {
      LEGO_LOGO
      translate y*(LEGO_KNOB_HEIGHT-EXTRA)
    }

    bounded_by { box { <-LEGO_KNOB_RADIUS, -LEGO_CORNER_SPACE-EXTRA, -LEGO_KNOB_RADIUS> <LEGO_KNOB_RADIUS, LEGO_KNOB_HEIGHT+LEGO_KNOB_CORNER_SPACE/5, LEGO_KNOB_RADIUS> } }
    
    translate <1, 0, 1> * LEGO_LENGTH / 2
  }


  #declare INV_INNER_KNOB = cylinder {
    <0, -EXTRA, 0>
    <0, LEGO_INV_KNOB_HEIGHT, 0>
    LEGO_INV_KNOB_RADIUS
    translate <1, 0, 1> *  LEGO_LENGTH/2
    translate -y * LEGO_CEILING_HEIGHT
  }


  
  #declare SCYLINDER1 = object {
    h_roundtube(LEGO_HEIGHT - LEGO_CEILING_HEIGHT + EXTRA, LEGO_SCYLINDER_RADIUS, LEGO_SCYLINDER_INNER_RADIUS)
    rotate z*90
    translate z*LEGO_LENGTH/2
  }


  #declare SCYLINDER3 = object {
    h_roundcylinder(3*LEGO_HEIGHT - LEGO_CEILING_HEIGHT + EXTRA, LEGO_SCYLINDER_RADIUS)
    rotate z*90
    translate z*l/2
  }


  #declare CYLINDER1 = object {
    h_roundtube(LEGO_HEIGHT - LEGO_CEILING_HEIGHT + EXTRA, LEGO_CYLINDER_RADIUS, LEGO_KNOB_RADIUS)
    rotate z*90
  }

  #declare CYLINDER3 = object {
    h_roundtube(3*LEGO_HEIGHT - LEGO_CEILING_HEIGHT + EXTRA, LEGO_CYLINDER_RADIUS, LEGO_KNOB_RADIUS)
    rotate z*90
  }


// Should not be used any more, use makro   INV_EDGE_CYLINDER (angle) instead
// macro not written yet.


#declare INV_CORNER_CYLINDER = difference {
    box { <0, 0, -2*LEGO_CORNER_SPACE> <2*LEGO_LENGTH, 2*LEGO_CORNER_SPACE, 2*LEGO_CORNER_SPACE> translate y*(LEGO_CORNER_SPACE/sqrt(2)+EXTRA)   rotate x*45 }
    cylinder { <-1, 0, 0> <2*LEGO_LENGTH+1, 0, 0> LEGO_CORNER_SPACE }
    translate -(y+z)*LEGO_CORNER_SPACE
}


#declare INV_CORNER_CYLINDER135 = difference {
    box {
        <0,0,0>
        <2*LEGO_LENGTH, LEGO_CORNER_SPACE, LEGO_CORNER_SPACE>
        translate -0.1*z*LEGO_CORNER_SPACE
        rotate x*22.5
        translate -z*LEGO_CORNER_SPACE*tan(pi/8)
    }
    cylinder {  <-1,0,0>  <2*LEGO_LENGTH+1,0,0>   LEGO_CORNER_SPACE translate <0, -1, -tan(pi/8)>*LEGO_CORNER_SPACE }
}



#end
