/*******************************************************************************

                      LEGO for POV-Ray 3.5
                     (C) Ernst Martin Witte

   This file is in no way related to the LEGO(tm) Group.
   It is provided for private non-commercial use only.


 *******************************************************************************/

#ifdef (__MOTOR_CYLINDER_INC)
#else

  #declare MOTOR_CYLINDER_INC = 1;
  
  #include "invroundbox.inc"
  #include "roundbox.inc"
  #include "invroundcylinder.inc"
  #include "roundcylinder.inc"
  
  
  #local br=LEGO_WALL_WIDTH;
  #local pr=LEGO_KNOB_RADIUS;
  #local hr=LEGO_CEILING_HEIGHT; //LEGO_TUBE_RADIUS;

  #local rrr = union {
    cylinder {  <br, br, br>  <2*l-br, br, br>    br  }
    intersection {
      cylinder { <0, br, br>  <2*l-br, br, br>    br  }
      cylinder { <br, br, 0>  <br, br, 2*l-br>    br  }
    }
  }
  
  #local corner = intersection {
    cylinder { -x*LEGO_WALL_WIDTH x*LEGO_WALL_WIDTH LEGO_WALL_WIDTH }
    cylinder { -z*LEGO_WALL_WIDTH z*LEGO_WALL_WIDTH LEGO_WALL_WIDTH }
    union {
      box { <-LEGO_WALL_WIDTH, -LEGO_LENGTH, -LEGO_WALL_WIDTH+LEGO_CORNER_SPACE>
	    < LEGO_WALL_WIDTH,  LEGO_LENGTH,  LEGO_WALL_WIDTH-LEGO_CORNER_SPACE> }
      box { <-LEGO_WALL_WIDTH+LEGO_CORNER_SPACE, -LEGO_LENGTH, -LEGO_WALL_WIDTH>
	    < LEGO_WALL_WIDTH-LEGO_CORNER_SPACE,  LEGO_LENGTH,  LEGO_WALL_WIDTH> }
      cylinder { <-LEGO_WALL_WIDTH+LEGO_CORNER_SPACE, -LEGO_LENGTH, -LEGO_WALL_WIDTH+LEGO_CORNER_SPACE>
	         <-LEGO_WALL_WIDTH+LEGO_CORNER_SPACE, +LEGO_LENGTH, -LEGO_WALL_WIDTH+LEGO_CORNER_SPACE>
	         LEGO_CORNER_SPACE }
    }
    translate <LEGO_WALL_WIDTH, LEGO_WALL_WIDTH, LEGO_WALL_WIDTH>
  }
  
  


  #local tempb = l - br - 1.0;
  #local tempa = sqrt (4*pr*pr + 3.2*3.2/tempb/tempb);
  #local temph = pr + sqrt(pr*pr-0.8*0.8) - 0.5;
  
  #local mzin = difference {
    union {
      intersection {
	box {
	  <-l, -l, -l>
	  <3*l, temph+1, 3*l>
	}
	
	object { roundcylinder(br, tempa)
	  rotate -y*90
	  scale <tempb/tempa,1,1>
	  translate <l,2*pr,l/2-br> }
	
      }
      box {
	<br, temph, l/2-br>
	<2*l-br, 4*h-hr, l/2>
      }
    }
    object { invroundcylinder(br,pr)
      rotate y*90 translate <l,pr,l/2> }

    object { invroundbox(br,pr+1,4)
      rotate y*90
      translate <l-2, -1, l/2> }
    object { h_invroundbox(3,4-2*EXTRA,l)
      rotate z*90
      translate <l+2+EXTRA, 0.3081, 0> }

    object { invroundbox(br,5*h,1.6)
      rotate y*90
      translate <l-0.8,-1,l/2> }

    object { invroundcylinder(br, 0.5)
      rotate y*90 translate <br+0.5,temph,l/2> }
    object { invroundcylinder(br, 0.5)
      rotate y*90 translate <2*l-br-0.5,temph,l/2> }
  }


  #declare MotorCylinder = union {
    difference {
      union {
	object {h_roundbox(2*l,4*h-LEGO_WALL_WIDTH,2*l) translate LEGO_WALL_WIDTH*y }
	object { corner rotate y *  00  translate <0, 0, 0> }
	object { corner rotate y *  90  translate <0, 0, 2*l> }
	object { corner rotate y * 180  translate <2*l, 0, 2*l> }
	object { corner rotate y * 270  translate <2*l, 0, 0> }
	cylinder { <LEGO_WALL_WIDTH, LEGO_WALL_WIDTH, LEGO_WALL_WIDTH>
	           <LEGO_LENGTH*2-LEGO_WALL_WIDTH, LEGO_WALL_WIDTH, LEGO_WALL_WIDTH>
	           LEGO_WALL_WIDTH }
	cylinder { <LEGO_WALL_WIDTH, LEGO_WALL_WIDTH, LEGO_LENGTH*2-LEGO_WALL_WIDTH>
	           <LEGO_LENGTH*2-LEGO_WALL_WIDTH, LEGO_WALL_WIDTH, LEGO_LENGTH*2-LEGO_WALL_WIDTH>
	           LEGO_WALL_WIDTH }
	object { h_roundcylinder(LEGO_LENGTH-3.5-LEGO_WALL_WIDTH+EXTRA, LEGO_WALL_WIDTH) 
	         rotate y*90 translate <LEGO_WALL_WIDTH, LEGO_WALL_WIDTH, LEGO_LENGTH-3.5+EXTRA> }
	object { h_roundcylinder(LEGO_LENGTH-3.5-LEGO_WALL_WIDTH+EXTRA, LEGO_WALL_WIDTH) 
	         rotate -y*90 translate <LEGO_WALL_WIDTH, LEGO_WALL_WIDTH, LEGO_LENGTH+3.5-EXTRA> }

	object { h_roundcylinder(LEGO_LENGTH-3.5-LEGO_WALL_WIDTH+EXTRA, LEGO_WALL_WIDTH) 
	         rotate y*90 translate <2*l-LEGO_WALL_WIDTH, LEGO_WALL_WIDTH, LEGO_LENGTH-3.5+EXTRA> }
	object { h_roundcylinder(LEGO_LENGTH-3.5-LEGO_WALL_WIDTH+EXTRA, LEGO_WALL_WIDTH) 
	         rotate -y*90 translate <2*l-LEGO_WALL_WIDTH, LEGO_WALL_WIDTH, LEGO_LENGTH+3.5-EXTRA> }
	
	object { TKNOB   translate <0, 4*h, 0> }
	object { TKNOB   translate <0, 4*h, l> }
	object { TKNOB   translate <l, 4*h, 0> }
	object { TKNOB   translate <l, 4*h, l> }
      }
      object { h_invroundbox(4*h-hr, 2*l-2*br, 2*l-2*br)
	rotate z*90
	translate <2*l-br,0,br>
      }
      object {
	invroundcylinder(br,3.5)
	translate <0,pr+br,l>	
      }
      object {
	invroundcylinder(br,3.5)
	translate <2*l-br,pr+br,l>	
      }

      object {
	invroundbox(LEGO_WALL_WIDTH,pr+br+1,7)
	translate <0,-1,l-3.5>
      }
      object {
	invroundbox(LEGO_WALL_WIDTH,pr+br+1,7)
	translate <2*l-LEGO_WALL_WIDTH,-1,l-3.5>
      }

    }

    object { mzin }
    object { mzin scale <1,1,-1> translate z*2*l }
    texture { LegoGray }
}

#end
