/*******************************************************************************

                      LEGO for POV-Ray 3.5
                     (C) Ernst Martin Witte

   This file is in no way related to the LEGO(tm) Group.
   It is provided for private non-commercial use only.

 *******************************************************************************/

#ifdef (__LEGO_SHOCK_ABSORBER_INC)
#else

  #declare __LEGO_SHOCK_ABSORBER_INC = 1;


  #include "helix.inc"
  
  #include "invroundbox.inc"
  #include "invroundcylinder.inc"
  #include "axles.inc"
  #include "roundbox.inc"
  


/*  #local    IDLE_Rounds = 2;
  #local    Rounds = 7;
  #local    FederDicke = 0.5;
  #local    FederTeilung = 16.0;         // Anzahl der Segmente, aus denen eine Umdrehung zusammengesetzt wird
  #local    FederDX_Const = 1 / (FederTeilung * sin(2*pi/FederTeilung) * (tri + FederDicke/2));

*/

  #declare __TEMP_SA_L_A = (3.25-0.5)*l;
  
  #local templ = __TEMP_SA_L_A;
  #declare ShockAbsorber_A = difference {
    union {
      object { roundcylinder(LEGO_LENGTH, LEGO_LENGTH/2) rotate y*90 translate z*LEGO_LENGTH/2 }
      difference {
	object { h_roundcylinder(templ, LEGO_LENGTH/2) rotate y*180 translate x*templ }
	object { h_invroundcylinder(LEGO_WALL_WIDTH/2, LEGO_TUBE_INLAY_RADIUS) rotate y*180 translate x*templ }
	object { invroundbox(LEGO_LENGTH-LEGO_CORNER_SPACE, 2, templ-l) rotate y*90 translate <0,-1,LEGO_LENGTH/2-LEGO_CORNER_SPACE/2> }
	object { __IAxle scale <3,1,1>}
	object { __INV_ROUND_AXLE translate x*(templ-LEGO_WALL_WIDTH/2)}
      }
    }
    object { h_invroundcylinder(LEGO_WALL_WIDTH/2, LEGO_TUBE_INLAY_RADIUS) rotate y*90 translate z*LEGO_LENGTH/2 }
    object { h_invroundcylinder(LEGO_WALL_WIDTH/2, LEGO_TUBE_INLAY_RADIUS) rotate y*90 translate z*LEGO_LENGTH/2 scale <1,1,-1> }
    object { invroundcylinder(LEGO_LENGTH-LEGO_WALL_WIDTH, LEGO_KNOB_RADIUS) rotate y*90 translate z*(LEGO_LENGTH/2-LEGO_WALL_WIDTH/2) }
    
    texture { LegoGray }
  }
  

  #declare __TEMP_SA_L_BA = 2.8;

  #declare ShockAbsorber_BA = difference {
    union {
      intersection {
	box {
	  <0, -LEGO_CROSSAXLE_WIDTH/2, -LEGO_KNOB_RADIUS>
	  <l-2.8+1,  LEGO_CROSSAXLE_WIDTH/2,  LEGO_KNOB_RADIUS>
	}
	cylinder {
	  <-l, 0, 0>
	  <2*l, 0, 0>
	  LEGO_KNOB_RADIUS
	}
      }
      intersection {
	object { roundbox(2.8, LEGO_CROSSAXLE_WIDTH, 6.2) translate <l-2.8, -LEGO_CROSSAXLE_WIDTH/2, -3.1> }
	cone {
	  <l-2.8, 0, 0>, 4.3
	  <l+0.01, 0, 0> 4.3-2.8
	}
      }
    }
    box {
      <-1, -LEGO_KNOB_RADIUS, -LEGO_CROSSAXLE_WIDTH/2>
      <l+1,  LEGO_KNOB_RADIUS,  LEGO_CROSSAXLE_WIDTH/2>
    }
  }
  

  #declare ShockAbsorber_B = difference {
    union {
      object { roundcylinder(LEGO_LENGTH, LEGO_LENGTH/2) rotate y*90 translate z*LEGO_LENGTH/2 }
      object { h_roundcylinder(LEGO_LENGTH/2,LEGO_LENGTH/2) rotate y*180 translate x*LEGO_LENGTH/2 }
      cylinder { x*LEGO_LENGTH/2 x*(LEGO_LENGTH/2+0.5) LEGO_KNOB_RADIUS - 0.5 }
      object { h_roundcylinder(LEGO_LENGTH/2+0.5+1.9,LEGO_KNOB_RADIUS) rotate y*180 translate x*(LEGO_LENGTH/2+0.5+1.9) }

      intersection {
	object { CrossAxle3 }
	box {  <-1, -l, -l>  <2.5*l, l,  l>
	}
	translate <l/2+0.5, 0, 0>
      }
      object {	ShockAbsorber_BA translate <l/2+0.5+2.5*l, 0, 0> }
    }
    
    object { h_invroundcylinder(LEGO_WALL_WIDTH/2, LEGO_TUBE_INLAY_RADIUS) rotate y*90 translate z*LEGO_LENGTH/2 }
    object { h_invroundcylinder(LEGO_WALL_WIDTH/2, LEGO_TUBE_INLAY_RADIUS) rotate y*90 translate z*LEGO_LENGTH/2 scale <1,1,-1> }
    object { invroundcylinder(LEGO_LENGTH-LEGO_WALL_WIDTH, LEGO_KNOB_RADIUS) rotate y*90 translate z*(LEGO_LENGTH/2-LEGO_WALL_WIDTH/2) }
   
    texture { LegoBlack }
  }
  

  

  #declare LEGO_SHOCKABSORBER_DELTA_LEN = __TEMP_SA_L_A - LEGO_LENGTH*1.5 - __TEMP_SA_L_BA;
  #declare LEGO_SHOCKABSORBER_MAX_LEN   = 5.5*LEGO_LENGTH;
  #declare LEGO_SHOCKABSORBER_MIN_LEN   = LEGO_SHOCKABSORBER_MAX_LEN - LEGO_SHOCKABSORBER_DELTA_LEN;
  
  #macro ShockAbsorber(percent)
  
  
  #if (percent > 1)
    #error concat ("shock absorber expansion > 1: ", str(percent, 4,4))
  #else #if (percent < 0)
    #error concat ("shock absorber expansion < 0: ", str(percent, 4,4))
  #end #end
  
  #local __TEMP_SA_HELIX_R2 = 0.5;
  #local __TEMP_SA_PL     = LEGO_SHOCKABSORBER_DELTA_LEN*(1-percent);
  #local __TEMP_SA_L      = LEGO_SHOCKABSORBER_DELTA_LEN*percent;
  
  #local __TEMP_SA_HL     = 5*LEGO_LENGTH - __TEMP_SA_L_A + LEGO_WALL_WIDTH/2 - __TEMP_SA_PL - (4+1)*__TEMP_SA_HELIX_R2;
  
  union {
    object { ShockAbsorber_A }
    object { ShockAbsorber_B  rotate y*180  translate x*(5.5*l - __TEMP_SA_PL) }
    object { helix(LEGO_KNOB_RADIUS+__TEMP_SA_HELIX_R2, __TEMP_SA_HELIX_R2, __TEMP_SA_HL, __TEMP_SA_HL/7, 16)
      translate x * (__TEMP_SA_L_A - LEGO_WALL_WIDTH/2 + 2*__TEMP_SA_HELIX_R2 )
      texture { T_Silver_3D }
    }
    object { helix(LEGO_KNOB_RADIUS+__TEMP_SA_HELIX_R2, __TEMP_SA_HELIX_R2, 2*__TEMP_SA_HELIX_R2, __TEMP_SA_HELIX_R2, 16)
      translate x * (__TEMP_SA_L_A - LEGO_WALL_WIDTH/2)
      texture { T_Silver_3D }
    }
    object { helix(LEGO_KNOB_RADIUS+__TEMP_SA_HELIX_R2, __TEMP_SA_HELIX_R2, 2*__TEMP_SA_HELIX_R2, __TEMP_SA_HELIX_R2, 16)
      translate x * (__TEMP_SA_L_A - LEGO_WALL_WIDTH/2 + 2*__TEMP_SA_HELIX_R2 + __TEMP_SA_HL )
      texture { T_Silver_3D }
    }
  }

  
#end

  
#end