/*******************************************************************************

                        LEGO for POV-Ray 3.5
                     (C) Ernst Martin Witte

   This file is in no way related to the LEGO(tm) Group.
   It is provided for private non-commercial use only.

   TECHBASE.INC: constants and textures for LEGO-Technic for POV-Ray

   Objects:      TKNOB :            Knob on LEGO-Technic bricks
                 LEGO_TUBE :        (massive) Tube for LEGO-Technic bricks
                 LEGO_INV_TUBE :    Object cut out of LEGO_TUBE
                 __ISector :
                 __IKranz :         Object cut out of some bushes
                 __IAxle :          An Lego Axle cut out of most bricks

 *******************************************************************************/

#ifdef (__LEGO_TBASE_INC)
#else

  #declare __LEGO_TBASE_INC = 1;


  #include "invroundbox.inc"

#declare LEGO_CROSSAXLE_WIDTH         = 1.8;

#declare LEGO_TUBE_RADIUS             = (LEGO_HEIGHT*3 - (LEGO_HEIGHT-LEGO_CEILING_HEIGHT)) / 2;     // outer radius of tubes in Technic Bricks
#declare LEGO_TUBE_INLAY_RADIUS       = LEGO_TUBE_RADIUS - LEGO_WALL_WIDTH/2;

#declare LEGO_RIVET_WALL_WIDTH        = 0.8;
#declare LEGO_RIVET_END_RADIUS        = LEGO_KNOB_RADIUS + 0.2;
#declare LEGO_RIVET_END_WIDTH         = 0.6;
#declare LEGO_GAP_LENGTH              = LEGO_HEIGHT;
#declare LEGO_GAP_WIDTH               = 0.8;

#declare LEGO_SPHERERIVET_RADIUS      = LEGO_LENGTH/2 - 1.0;
#declare LEGO_SPHERERIVET_CENTER      = 14.3 - l - LEGO_SPHERERIVET_RADIUS;


#declare LEGO_BUSH_RADIUS             = LEGO_KNOB_RADIUS + 0.4;
#declare LEGO_BUSH_END_RADIUS         = LEGO_TUBE_RADIUS;
#declare LEGO_BUSH_END_WIDTH          = 1.2;

#declare LEGO_TOOTH_HEIGHT            = 2.1;
#declare LEGO_GEAR8_RADIUS            = (  LEGO_LENGTH - LEGO_TOOTH_HEIGHT - 0.2)/2;
#declare LEGO_GEAR16_RADIUS           = (2*LEGO_LENGTH - LEGO_TOOTH_HEIGHT - 0.2)/2;
#declare LEGO_GEAR24_RADIUS           = (3*LEGO_LENGTH - LEGO_TOOTH_HEIGHT - 0.2)/2;
#declare LEGO_GEAR28_RADIUS           = (3.5375*LEGO_LENGTH - LEGO_TOOTH_HEIGHT - 0.2)/2;
#declare LEGO_GEAR40_RADIUS           = (5*LEGO_LENGTH - LEGO_TOOTH_HEIGHT - 0.2)/2;



#declare TKNOB = union {
    difference {
        union {
            cylinder {
                <0, -EXTRA, 0>
                <0, LEGO_KNOB_HEIGHT, 0>
                LEGO_KNOB_RADIUS-LEGO_KNOB_CORNER_SPACE
            }
            cylinder {
                <0, -2*EXTRA, 0>
                <0, LEGO_KNOB_HEIGHT-LEGO_KNOB_CORNER_SPACE, 0>
                LEGO_KNOB_RADIUS
            }
        }
        cylinder {
            <0, -3*EXTRA, 0>
            <0, LEGO_KNOB_HEIGHT+EXTRA, 0>
            LEGO_KNOB_INNER_RADIUS
        }
        cylinder {
            <0, LEGO_KNOB_HEIGHT-LEGO_CORNER_SPACE, 0>
            <0, LEGO_KNOB_HEIGHT+EXTRA, 0>
            LEGO_KNOB_INNER_RADIUS+LEGO_CORNER_SPACE
        }
    }
    torus {
        LEGO_KNOB_INNER_RADIUS+LEGO_CORNER_SPACE
        LEGO_CORNER_SPACE
        translate y * (LEGO_KNOB_HEIGHT-LEGO_CORNER_SPACE)
    }
    torus {
        LEGO_KNOB_RADIUS-LEGO_KNOB_CORNER_SPACE
        LEGO_KNOB_CORNER_SPACE
        translate y * (LEGO_KNOB_HEIGHT-LEGO_KNOB_CORNER_SPACE)
    }
    object {
        LEGO_LOGO
        scale y+(x+z)*LEGO_KNOB_INNER_RADIUS/LEGO_KNOB_RADIUS
    }

    bounded_by { box { <-LEGO_KNOB_RADIUS, -EXTRA, -LEGO_KNOB_RADIUS> <LEGO_KNOB_RADIUS, LEGO_KNOB_HEIGHT, LEGO_KNOB_RADIUS> } }
    translate (x+z) * l / 2
}




  #declare LEGO_INV_TUBE = union {
    object { invroundcylinderX(LEGO_LENGTH-LEGO_WALL_WIDTH, LEGO_KNOB_RADIUS, LEGO_CORNER_SPACE/2)
      rotate -y*90 translate z*LEGO_WALL_WIDTH/2 }
    object { h_invroundcylinderX(LEGO_WALL_WIDTH/2, LEGO_TUBE_INLAY_RADIUS, LEGO_CORNER_SPACE/2)
      rotate -y*90 }
    object { h_invroundcylinderX(LEGO_WALL_WIDTH/2, LEGO_TUBE_INLAY_RADIUS, LEGO_CORNER_SPACE/2)
      rotate y*90 translate z*LEGO_LENGTH}
      
/*    cylinder {
        <0, 0, 0>
        <0, 0, l>
        LEGO_KNOB_RADIUS
    }
    #local i = 0;
    #while (i < 2)
        difference {
            union {
                cylinder {
                    <0, 0, -1*EXTRA>
                    <0, 0, LEGO_WALL_WIDTH/2 + LEGO_CORNER_SPACE / 2>
                    LEGO_KNOB_RADIUS + LEGO_CORNER_SPACE / 2
                }
                cylinder {
                    <0, 0, -2*EXTRA>
                    <0, 0, LEGO_WALL_WIDTH/2>
                    LEGO_TUBE_INLAY_RADIUS
                }
                cylinder {
                    <0, 0, -3*EXTRA>
                    <0, 0, LEGO_CORNER_SPACE / 2>
                    LEGO_TUBE_INLAY_RADIUS + LEGO_CORNER_SPACE/2
                }
            }
            torus {
                LEGO_KNOB_RADIUS + LEGO_CORNER_SPACE/2
                LEGO_CORNER_SPACE/2
                rotate x * 90
                translate z*(LEGO_WALL_WIDTH/2 + LEGO_CORNER_SPACE/2)
            }
            torus {
                LEGO_TUBE_INLAY_RADIUS + LEGO_CORNER_SPACE/2
                LEGO_CORNER_SPACE/2
                rotate x * 90
                translate z*LEGO_CORNER_SPACE/2
            }
            scale <1, 1, 1-2*i>
            translate z*i*LEGO_LENGTH
        }
        #local i = i + 1;
    #end

    bounded_by { box { (x+y)*(- LEGO_TUBE_INLAY_RADIUS - LEGO_CORNER_SPACE/2)
                       (x+y)*(  LEGO_TUBE_INLAY_RADIUS + LEGO_CORNER_SPACE/2) + z*LEGO_LENGTH }}

*/
    translate <0, 3*LEGO_HEIGHT - LEGO_TUBE_RADIUS, 0>
}



#declare LEGO_TUBE = union {
    cylinder {
        <0, 0, LEGO_WALL_WIDTH - EXTRA>
        <0, 0, LEGO_LENGTH - LEGO_WALL_WIDTH + EXTRA>
        LEGO_TUBE_RADIUS
    }
    box {
        <-LEGO_TUBE_RADIUS, 0, LEGO_WALL_WIDTH - 2*EXTRA>
        <+LEGO_TUBE_RADIUS, LEGO_TUBE_RADIUS-LEGO_CEILING_HEIGHT+EXTRA, LEGO_LENGTH - LEGO_WALL_WIDTH + EXTRA>
    }
    translate <0, 3*LEGO_HEIGHT - LEGO_TUBE_RADIUS, 0>
}


  ////////////////////////////////////////////////////////////////////////
  
  
  #macro __ISectorX(Height)
    
    intersection {
      object { h_invroundboxX(Height, LEGO_LENGTH, LEGO_LENGTH, LEGO_CORNER_SPACE/2) 
	rotate -z*90 translate y*Height}
      object { h_invroundboxX(Height, LEGO_LENGTH, LEGO_LENGTH, LEGO_CORNER_SPACE/2) 
	rotate -z*90 translate y*(Height-EXTRA)
	rotate y*(90-180/16*1.05) }
      
      rotate y*(180/16*0.025)
    }  
    
  #end
  
  
  #macro __IKranzX(Height)
    
    union {
      #local i = 0;
      #while (i < 16)
	object { __ISectorX(Height) rotate y*i*360/16 }
	#local i = i+1;
      #end
    }
    
  #end

  #declare __ISector = __ISectorX(LEGO_HEIGHT)
  #declare __IKranz = __IKranzX(LEGO_HEIGHT)

  ////////////////////////////////////////////////////////////////////////

#declare __IAxle = difference {
    intersection {
        union {
            box { <-LEGO_LENGTH-EXTRA, -LEGO_KNOB_RADIUS, -LEGO_CROSSAXLE_WIDTH/2> <LEGO_LENGTH+EXTRA, LEGO_KNOB_RADIUS, LEGO_CROSSAXLE_WIDTH/2> }
            box { <-LEGO_LENGTH, -LEGO_CROSSAXLE_WIDTH/2, -LEGO_KNOB_RADIUS> <LEGO_LENGTH, LEGO_CROSSAXLE_WIDTH/2, LEGO_KNOB_RADIUS> }
            box {(-x*(LEGO_LENGTH-EXTRA)-(y+z)*(LEGO_CROSSAXLE_WIDTH/2 + LEGO_KNOB_CORNER_SPACE))
                  (x*(LEGO_LENGTH-EXTRA)+(y+z)*(LEGO_CROSSAXLE_WIDTH/2 + LEGO_KNOB_CORNER_SPACE))
                }
        }
        cylinder { <-LEGO_LENGTH-2*EXTRA, 0, 0> <LEGO_LENGTH+2*EXTRA, 0, 0> LEGO_KNOB_RADIUS }

    }

    #local i = 0;
    #while (i < 4)
        cylinder { -x*(LEGO_LENGTH+3*EXTRA)-(y+z)*(LEGO_CROSSAXLE_WIDTH/2 + LEGO_KNOB_CORNER_SPACE)
                    x*(LEGO_LENGTH+3*EXTRA)-(y+z)*(LEGO_CROSSAXLE_WIDTH/2 + LEGO_KNOB_CORNER_SPACE)
                    LEGO_KNOB_CORNER_SPACE
                    rotate x*90*i
                 }
        #local i = i+1;
    #end

    bounded_by { box { <-LEGO_LENGTH-4*EXTRA, -LEGO_KNOB_RADIUS, -LEGO_KNOB_RADIUS>, <LEGO_LENGTH+4*EXTRA, LEGO_KNOB_RADIUS, LEGO_KNOB_RADIUS> } }
}

  
  
  #declare __INV_ROUND_AXLE = intersection {
    difference {
      #local    a = LEGO_CORNER_SPACE*2/2;
      #local   c0 = -(y+z)*(LEGO_CROSSAXLE_WIDTH/2 + LEGO_KNOB_CORNER_SPACE-EXTRA);
      union {                        
	box { <0, -LEGO_KNOB_RADIUS-a+EXTRA, -LEGO_CROSSAXLE_WIDTH/2-a+EXTRA> <2*LEGO_CORNER_SPACE, LEGO_KNOB_RADIUS+a+EXTRA, LEGO_CROSSAXLE_WIDTH/2+a-EXTRA> }
	box { <0, -LEGO_CROSSAXLE_WIDTH/2-a+EXTRA, -LEGO_KNOB_RADIUS-a+EXTRA> <2*LEGO_CORNER_SPACE, LEGO_CROSSAXLE_WIDTH/2+a+EXTRA, LEGO_KNOB_RADIUS+a-EXTRA> }
	box {c0, x*2*LEGO_CORNER_SPACE-c0}
      }
      #local i = 0;
      #while (i < 4)
	union {
	  cylinder { c0 - x,
	    c0 + x*(2*LEGO_CORNER_SPACE + 1),
	    LEGO_KNOB_CORNER_SPACE-a-EXTRA
	  }
	  torus {
	    LEGO_KNOB_CORNER_SPACE-a-EXTRA
	    a
	    rotate z*90
	    translate c0
	  }
	  cylinder {
	    <0, LEGO_KNOB_RADIUS+1, LEGO_CROSSAXLE_WIDTH/2+a-EXTRA>
	    <0, LEGO_CROSSAXLE_WIDTH/2 + LEGO_KNOB_CORNER_SPACE-EXTRA, LEGO_CROSSAXLE_WIDTH/2+a-EXTRA>
	    a
	  }
	  cylinder {
	    <0, LEGO_KNOB_RADIUS+1, -LEGO_CROSSAXLE_WIDTH/2-a+EXTRA>
	    <0, LEGO_CROSSAXLE_WIDTH/2 + LEGO_KNOB_CORNER_SPACE-EXTRA, -LEGO_CROSSAXLE_WIDTH/2-a+EXTRA>
	    a
	  }
	  rotate x*90*i
	}
	#local i = i+1;
      #end
      torus {
	LEGO_KNOB_RADIUS+a-EXTRA
	a
	rotate z*90            
      }
    }
    cylinder { <-2, 0, 0> <a*4, 0, 0> LEGO_KNOB_RADIUS+a-EXTRA }
    bounded_by { cylinder { <-2, 0, 0> <a*4, 0, 0> LEGO_KNOB_RADIUS+a } } 
    translate -x*(a+EXTRA)
  }

#end





