/*******************************************************************************

                      LEGO for POV-Ray 3.5
                     (C) Ernst Martin Witte

   This file is in no way related to the LEGO(tm) Group.
   It is provided for private non-commercial use only.


 *******************************************************************************/

#ifdef (__LEGO_UNIVERSAL_JOINT)
#else

  #declare __LEGO_UNIVERSAL_JOINT = 1;
  

  #include "invroundbox.inc"
  #include "roundcylinder.inc"
  #include "roundbox.inc"
  #include "invroundcylinder.inc"
  

// Definition eines Kardan-Gelenkes durch die drei beweglichen Einzelteile


  #declare UniversalJoint_A = difference {
    union {
      difference {
	cylinder { <0, 0, 0> <LEGO_LENGTH, 0, 0>  LEGO_BUSH_RADIUS }

	object { 
	  invroundbox(LEGO_BUSH_RADIUS*2, LEGO_GAP_WIDTH, LEGO_LENGTH - 2*LEGO_BUSH_END_WIDTH)
	  rotate y*90
	  translate <LEGO_BUSH_END_WIDTH, -LEGO_GAP_WIDTH/2, LEGO_BUSH_RADIUS>
	}
	  
      }
      
      object {
	h_roundcylinder (0.75*LEGO_LENGTH + LEGO_BUSH_END_WIDTH, LEGO_BUSH_END_RADIUS)
	translate x*(LEGO_LENGTH-LEGO_BUSH_END_WIDTH)
      }

      difference {
	object { roundcylinder(LEGO_BUSH_END_WIDTH, LEGO_BUSH_END_RADIUS) }
	object { invroundcylinder(LEGO_BUSH_END_WIDTH, LEGO_KNOB_RADIUS) translate <0, -l/2, -l/2> }
	object { invroundcylinder(LEGO_BUSH_END_WIDTH, LEGO_KNOB_RADIUS) translate <0, -l/2, +l/2> }
	object { invroundcylinder(LEGO_BUSH_END_WIDTH, LEGO_KNOB_RADIUS) translate <0, +l/2, -l/2> }
	object { invroundcylinder(LEGO_BUSH_END_WIDTH, LEGO_KNOB_RADIUS) translate <0, +l/2, +l/2> }
      }

      intersection {
	union {
	  object { roundcylinder(LEGO_BUSH_END_RADIUS-2.5, l/4) rotate +y*90 translate <2*l, 0, LEGO_BUSH_END_RADIUS> }
	  object { roundcylinder(LEGO_BUSH_END_RADIUS-2.5, l/4) rotate +y*90 translate <2*l, 0, LEGO_BUSH_END_RADIUS> scale <1,1,-1>}
	  object { roundbox(l/2, l/2, LEGO_BUSH_END_RADIUS-2.5) translate <1.5*l, -l/4, -LEGO_BUSH_END_RADIUS> }
	  object { roundbox(l/2, l/2, LEGO_BUSH_END_RADIUS-2.5) translate <1.5*l, -l/4, -LEGO_BUSH_END_RADIUS> scale <1,1,-1>}
	}
	cylinder { <LEGO_LENGTH, 0, 0> <3*LEGO_LENGTH, 0, 0> LEGO_BUSH_END_RADIUS }
      }
    }
    object { __IAxle }
    object { __INV_ROUND_AXLE rotate y*180}
    cylinder { <0, 0, 0> <2*LEGO_LENGTH, 0, 0> sqrt(2)*LEGO_CROSSAXLE_WIDTH/2 }


    
    object { 
      invroundcylinder(2*LEGO_BUSH_END_RADIUS, 2.5)
      rotate z*90
      translate <13.5, -LEGO_BUSH_END_RADIUS, 0>
    }
    
  
    box { <13.5, -LEGO_LENGTH, -2.5> <1.75*LEGO_LENGTH+EXTRA, LEGO_LENGTH, 2.5> }
    
    box { <0, 0, -l> <l, l, l>  rotate z*45  translate <1.75*l, l/4, 0> }    
    box { <0, 0, -l> <l, l, l>  rotate z*45  translate <1.75*l, l/4, 0> scale <1, -1, 1> }
  
    object { invroundcylinder(LEGO_BUSH_END_RADIUS-2.5, 0.7) rotate -y*90 translate <2*LEGO_LENGTH, 0, -LEGO_BUSH_END_RADIUS> }
    object { invroundcylinder(LEGO_BUSH_END_RADIUS-2.5, 0.7) rotate  y*90 translate <2*LEGO_LENGTH, 0,  LEGO_BUSH_END_RADIUS> }

    texture { LegoGray }

  }


  #declare ___temp = union {
    object { roundcylinder (5, LEGO_LENGTH/5) rotate z*90 translate -y*2.5 }
    object { roundcylinder (2*LEGO_BUSH_END_RADIUS, 0.65) rotate z*90 translate -y*LEGO_BUSH_END_RADIUS }
  }
  
  

  #declare UniversalJoint_M = union {  
    object { roundcylinder(LEGO_LENGTH/2, 2.5) translate <-LEGO_LENGTH/4, 0, 0> }
    
    object { ___temp }
    object { ___temp rotate x*90}
    
    texture { LegoGray }
  }



  // Usage :
  //
  //     Axis1_Rotation: 0..360    around x-Axis
  //     Axis2_Angle:    -89 .. 89 around z-Axis, 0 == positive x-Axis
  //
  // Output:
  //     Axis2_Rotation
  //        1) rotate Axis2 by x*Axis2_Rotation
  //        2) rotate Axis2 by z*Axis2_Angle

  #macro UniversalJoint_Axis2Rot (Axis1_Rotation, Axis2_Angle)
    
    #local __phi = Axis1_Rotation * pi/180;
    #local __decl = Axis2_Angle * pi/180;

    #local __tmp = <-sin(__decl)*cos(__phi), cos(__decl)*cos(__phi), sin(__phi)*cos(__decl)>;
    #local __beta = -180/pi*atan2(__tmp.x, sqrt(__tmp.y*__tmp.y+__tmp.z*__tmp.z));
    
    #local __aaaa = vaxis_rotate(vaxis_rotate(x, z, __beta), x, Axis1_Rotation );
    #local __bbbb = vaxis_rotate(x, z, Axis2_Angle );
    #local __omega = 180/pi*acos(vdot (__aaaa, __bbbb)/vlength(__aaaa)/vlength(__bbbb));
  
  
    #local  __vx = vcross (__aaaa, __bbbb);
    #if (vlength(__vx) > 0)
      #local __vx = vnormalize(__vx);
    #end
  
    #if (vlength(__vx + vaxis_rotate(vaxis_rotate(y, z, __beta), x, Axis1_Rotation)) = 0)
      #local __omega = -__omega;
    #end
  
    #local __zz = vaxis_rotate(vaxis_rotate (vaxis_rotate (z, y, __omega), z, __beta), x, Axis1_Rotation);
    #local __zz = vaxis_rotate(__zz, z, -Axis2_Angle);
    
    #local Axis2_Rotation = -180/pi*atan2(__zz.y, __zz.z);
  

    Axis2_Rotation
  #end
  
  
  

  // Axis1_Rotation: 0..360    around x-Axis
  // Axis2_Angle:    -89 .. 89 around z-Axis, 0 == positive x-Axis

  #macro UniversalJoint(Axis1_Rotation, Axis2_Angle)
  
  // Usage :
  //     __omega + __beta
  //        1) rotate Axis2 by y*omega
  //        2) rotate Axis2 by z*beta
  //        3) rotate Axis2 by x*Axis1_Rotation
  
  // OR 
  //     Axis2_Rotation
  //        1) rotate Axis2 by x*Axis2_Rotation
  //        2) rotate Axis2 by z*Axis2_Angle

  #local __phi = Axis1_Rotation * pi/180;
  #local __decl = Axis2_Angle * pi/180;

  #local __tmp = <-sin(__decl)*cos(__phi), cos(__decl)*cos(__phi), sin(__phi)*cos(__decl)>;
  #local __beta = -180/pi*atan2(__tmp.x, sqrt(__tmp.y*__tmp.y+__tmp.z*__tmp.z));

  #local __aaaa = vaxis_rotate(vaxis_rotate(x, z, __beta), x, Axis1_Rotation );
  #local __bbbb = vaxis_rotate(x, z, Axis2_Angle );
  #local __omega = 180/pi*acos(vdot (__aaaa, __bbbb)/vlength(__aaaa)/vlength(__bbbb));
  
  
  #local  __vx = vcross (__aaaa, __bbbb);
  #if (vlength(__vx) > 0)
    #local __vx = vnormalize(__vx);
  #end
  
  #if (vlength(__vx + vaxis_rotate(vaxis_rotate(y, z, __beta), x, Axis1_Rotation)) = 0)
    #local __omega = -__omega;
  #end
  
  #local __zz = vaxis_rotate(vaxis_rotate (vaxis_rotate (z, y, __omega), z, __beta), x, Axis1_Rotation);
  #local __zz = vaxis_rotate(__zz, z, -Axis2_Angle);
  
  #local Axis2_Rotation = -180/pi*atan2(__zz.y, __zz.z);


  union {
    object {
      UniversalJoint_A
      translate -x*2*l
      rotate x*Axis1_Rotation
    }
    object {
      UniversalJoint_M
      rotate z*__beta
      rotate x*Axis1_Rotation
    }
    object {
      UniversalJoint_A
      rotate y*180
      rotate x*90

      translate x*2*l

      rotate y*__omega
      rotate z*__beta
      rotate x*Axis1_Rotation
    }
  }

  
#end
 
  

#end