/*******************************************************************************

                      LEGO for POV-Ray 3.5
                     (C) Ernst Martin Witte

   This file is in no way related to the LEGO(tm) Group.
   It is provided for private non-commercial use only.

 *******************************************************************************/

#ifdef (__LEGO_WHEELS_INC )
#else

  #declare __LEGO_WHEELS_INC = 1;

  
  #include "roundcylinder.inc"
  #include "invroundcylinder.inc"
  #include "roundbox.inc"
  #include "invroundbox.inc"
  #include "inv_round_edge.inc"
  #include "invround_coneedge.inc"
  

  #declare   WheelSmallThick = difference {
    
    #local __eps = 180-atan2(2.5/2, 1.2) * 180/pi;
    
    union {

      difference {
	union {
	  cylinder { <0, 0, 0> <LEGO_LENGTH, 0, 0> 14.3 / 2 }
	  
	  object { h_roundcylinder(1.2, 16.8/2) }
	  object { h_roundcylinder(1.2, 16.8/2) rotate y*180 translate x*LEGO_LENGTH }
	  cone { <1.2, 0, 0> 16.8/2  <(LEGO_LENGTH-3.2)/2, 0, 0> 14.3/2 }
	  cone { <LEGO_LENGTH-1.2, 0, 0> 16.8/2  <LEGO_LENGTH-(LEGO_LENGTH-3.2)/2, 0, 0> 14.3/2 }

	  object { invround_coneedge(14.3/2, __eps, -90) rotate z*90 translate x*(LEGO_LENGTH-2.4) }
	  object { invround_coneedge(14.3/2, __eps, -90) rotate z*90 scale <-1,1,1> translate x*2.4 }
	}
	invroundcylinder(LEGO_LENGTH, 11.8/2)
	object { invround_coneedge(16.8/2, __eps, 90) rotate z*90 translate x*(LEGO_LENGTH-1.2) }
	object { invround_coneedge(16.8/2, __eps, 90) rotate z*90 scale <-1,1,1> translate x*1.2 }
      }
      roundcylinder(LEGO_LENGTH, 7.8/2)
      object { roundbox(LEGO_LENGTH, 14, 2.4) translate <0, -7, -1.2> }
    }
    object { __IAxle }
    object { __INV_ROUND_AXLE translate x*LEGO_LENGTH }
    object { __INV_ROUND_AXLE rotate y*180 }
    object { invroundbox(LEGO_LENGTH, LEGO_CROSSAXLE_WIDTH+EXTRA*2, 8) translate <0, -LEGO_CROSSAXLE_WIDTH/2-EXTRA, -4> }
    texture { LegoGray }
  }


  #declare WheelMediumThin = difference {
    union {
      difference {
	roundcylinder(LEGO_LENGTH/2, 23.9/2)

	cone { <-EXTRA, 0, 0> 20.6/2 <0.7, 0, 0> 18.8/2 }
	cone { <l/2+EXTRA, 0, 0> 20.6/2 <l/2-0.7, 0, 0> 18.8/2 }
	
	#local __PHI = 90 + atan2(0.9, 0.7)*180/pi;
	object { invround_coneedge(20.6/2, __PHI, -(180-__PHI)) rotate z*90  scale <-1,1,1> }
	object { invround_coneedge(20.6/2, __PHI, -(180-__PHI)) rotate z*90  translate x*LEGO_LENGTH/2 }
	
	torus{ 23.9/2   1  rotate z*90  translate x*l/4 }
      }
      
      object { invround_coneedge(18.8/2, __PHI, 0) rotate z*90 translate x*0.7 }
      object { invround_coneedge(18.8/2, __PHI, 0) rotate z*90 scale <-1,1,1> translate x*(LEGO_LENGTH/2-0.7) }
      
      roundcylinder (LEGO_LENGTH/2, 6.4/2 )
    }

    #local i=0;
    #while (i<6)
      union {
	object { invroundcylinder(LEGO_LENGTH/2-1.4, LEGO_KNOB_RADIUS) translate <0.7, LEGO_LENGTH, 0> }
	object { h_invroundcylinder(0.7-EXTRA, 5.8/2) translate y*LEGO_LENGTH }
	object { h_invroundcylinder(0.7-EXTRA, 5.8/2) rotate y*180 translate <LEGO_LENGTH/2, LEGO_LENGTH, 0> }
	
	    rotate x*60*i
      }
      #local i=i+1;
    #end
    
    
    object { __IAxle }
    object { __INV_ROUND_AXLE translate x*LEGO_LENGTH/2 }
    object { __INV_ROUND_AXLE rotate y*180 }
    
    object { invroundbox(LEGO_LENGTH/2, 0.8, 9) translate <0,-0.4, -4.5> }
    difference {
      object { invroundbox(LEGO_LENGTH/2-1.4, 0.8-EXTRA*2, 9) translate <0.7,-0.4-EXTRA, -4.5> }
      cylinder { <-1, 0, 0> <LEGO_LENGTH, 0, 0> 6.4/2 }
    }

    object { invroundcylinder(LEGO_LENGTH/2-1.4, 0.4) translate <0.7, 0, -4.5> }
    object { invroundcylinder(LEGO_LENGTH/2-1.4, 0.4) translate <0.7, 0,  4.5> }
    
    texture { LegoGray }
  }


  #declare WheelBigThin = difference {
    #local __WI = 2*sqrt(3);
    
    union {
      intersection {
	union {
	  torus {16, 2 rotate z*90 translate x*l/2}
	  cylinder { <l/2, 0, -16> <l/2, 0, 16> 2 }
	}
	union {
	  cone { <l/2, 0, 0> 16 <l/2-__WI/2, 0, 0> 16+__WI }
	  cone { <l/2, 0, 0> 16 <l/2+__WI/2, 0, 0> 16+__WI }
	}
      }
      roundcylinder(LEGO_LENGTH, 7.8/2)
      object { roundcylinder(6.2, 3.2) translate  <LEGO_LENGTH/2-3.1, 0,  LEGO_LENGTH*3/2> }
      object { roundcylinder(6.2, 3.2) translate  <LEGO_LENGTH/2-3.1, 0, -LEGO_LENGTH*3/2> }
    }
    object { invroundcylinder(6.2, LEGO_KNOB_RADIUS) translate  <LEGO_LENGTH/2-3.1, 0,  LEGO_LENGTH*3/2> }
    object { invroundcylinder(6.2, LEGO_KNOB_RADIUS) translate  <LEGO_LENGTH/2-3.1, 0, -LEGO_LENGTH*3/2> }
    
    object { __IAxle }
    object { __INV_ROUND_AXLE translate x*LEGO_LENGTH }
    object { __INV_ROUND_AXLE rotate y*180 }
    
    difference {
      object { invroundbox(__WI, 0.8, 13) translate <LEGO_LENGTH/2-__WI/2, -0.4, -6.5> }
      cylinder{<-1, 0, 0> <LEGO_LENGTH+1, 0, 0> 7.8/2 + EXTRA }      
    }

    object { invroundcylinder(__WI, 0.4) translate <LEGO_LENGTH/2-__WI/2, 0,  6.5> }
    object { invroundcylinder(__WI, 0.4) translate <LEGO_LENGTH/2-__WI/2, 0, -6.5> }

    
    texture { LegoGray }
  }



  // Liegt waagerecht, Mitte(Aufsatz fr Achse) = <0,0,0>
  
  #declare LEGO_STEERINGWHEEL_TORUS_RADIUS = 4.5/2;
  #declare LEGO_STEERINGWHEEL_RADIUS       = 5*LEGO_LENGTH/2-LEGO_STEERINGWHEEL_TORUS_RADIUS;
  
  #local r2 = sqrt(2.5)*LEGO_LENGTH;
  #local r1 = r2 + LEGO_STEERINGWHEEL_TORUS_RADIUS+LEGO_HEIGHT; 
  
  #local __phi1 = 180/pi*asin(LEGO_LENGTH/2 / r1);
  #local __phi2 = 180/pi*asin(LEGO_LENGTH/2 / r2);
  #local __phi1I = 180/pi*asin(LEGO_LENGTH/2 / (r1-LEGO_HEIGHT));
  #local __phi2I = 180/pi*asin(LEGO_LENGTH/2 / (r2-LEGO_HEIGHT));
  #local __eps  = 180/pi*atan2(r1-r2, r1*cos(__phi1*pi/180)-r2*cos(__phi2*pi/180));
  #local __epsI  = 180/pi*atan2(r1-r2, (r1-LEGO_HEIGHT)*cos(__phi1I*pi/180)-(r2-LEGO_HEIGHT)*cos(__phi2I*pi/180));

  #declare SteeringWheel = difference {
    union { 
      difference {
	union {
	  torus {
	    LEGO_STEERINGWHEEL_RADIUS
	    LEGO_STEERINGWHEEL_TORUS_RADIUS
	  }
  
	  difference {
	    intersection {
	      union {
		difference {
		  cone {
		    <0, -LEGO_STEERINGWHEEL_TORUS_RADIUS - LEGO_HEIGHT, 0> r2
		    <0, 0, 0> r1
		  }
		  cone {
		    <0,  LEGO_STEERINGWHEEL_TORUS_RADIUS,               0>, 
		    r1 - LEGO_HEIGHT + LEGO_STEERINGWHEEL_TORUS_RADIUS
		    <0, -LEGO_STEERINGWHEEL_TORUS_RADIUS-2*LEGO_HEIGHT, 0>, 
		    r2 - LEGO_HEIGHT*2
		  }
		}
		difference {
		  object {
		    invround_coneedge(r2, 135, 0)
		    translate -y*(LEGO_STEERINGWHEEL_TORUS_RADIUS)
		  }
		  #local i = 0;
		  #while (i < 4)
		    object { inv_roundedge(4*r1, 90) translate <-2*r1, -LEGO_STEERINGWHEEL_TORUS_RADIUS, -LEGO_LENGTH/2> rotate y*i*90}
		    #local i=i+1;
		  #end
		}
	      }
	      
	      union {
		box {
		  <-LEGO_STEERINGWHEEL_RADIUS, LEGO_LENGTH,   -LEGO_LENGTH/2>
		  < LEGO_STEERINGWHEEL_RADIUS, -2*LEGO_LENGTH, LEGO_LENGTH/2>
		}
		box {
		  <-LEGO_LENGTH/2, LEGO_LENGTH,    0>
		  < LEGO_LENGTH/2, -2*LEGO_LENGTH, LEGO_STEERINGWHEEL_RADIUS>
		}
	      }
	    }
	    object { 
	      invround_coneedge(r2, 135, 0)
	      translate -y*(LEGO_STEERINGWHEEL_TORUS_RADIUS+LEGO_HEIGHT)
	    }
	    object { 
	      invround_coneedge(r1 - LEGO_HEIGHT, 135, 180)
	    }
	    
	    
	    #local __tmp = object {
	      inv_roundedge(LEGO_LENGTH*3, __phi1+90)
	      rotate -x*__phi1
	      translate -x*LEGO_LENGTH
	      rotate -z*(180-__eps)
	      translate <r1*cos(__phi1*pi/180), 0, -LEGO_LENGTH/2>
	    }
	    
	    #local __tmp2 = union {
	      object { __tmp }
	      object { __tmp scale <1,1,-1> }
	    }
	    
	    
	    
	    #local __tmpI = object {
	      inv_roundedge(LEGO_LENGTH*3, -__phi1I+90)
	      rotate x*__phi1I
	      translate -x*LEGO_LENGTH*2
	      rotate z*__epsI
	      translate <(r1-LEGO_HEIGHT)*cos(__phi1I*pi/180), 0, -LEGO_LENGTH/2>
	    }
	    
	    #local __tmpI2 = union {
	      object { __tmpI }
	      object { __tmpI scale <1,1,-1> }
	    }
	    
	    
	    object { __tmp2 }
	    object { __tmp2 rotate y*180 }
	    object { __tmp2 rotate -y*90 }
	    
	    object { __tmpI2 }
	    object { __tmpI2 rotate y*180 }
	    object { __tmpI2 rotate -y*90 }
	    
	    #local i = 0;
	    #while (i < 4)
	      object { inv_roundedge(4*r1, 90) translate <-2*r1, 0, -LEGO_LENGTH/2> rotate y*i*90}
	      object { inv_roundedge(4*r1, 90) translate <-2*r1, 0, -LEGO_LENGTH/2> scale <1,-1,1> translate -y*(r1-r2) rotate y*i*90 }
	      #local i=i+1;
	    #end
	    
	  }
	  
	  
	  object { roundbox(r2*2+4*LEGO_CORNER_SPACE, LEGO_HEIGHT/2, LEGO_LENGTH) translate <-r2-2*LEGO_CORNER_SPACE, -(r1-r2)+LEGO_HEIGHT/2, -LEGO_LENGTH/2> }
	  object { roundbox(r2*2-LEGO_HEIGHT, LEGO_HEIGHT*2/3, LEGO_LENGTH) translate <-r2+LEGO_HEIGHT/2, -(r1-r2), -LEGO_LENGTH/2> }
	  object { roundbox(r2, LEGO_HEIGHT/2, LEGO_LENGTH) translate <-r2, -(r1-r2)+LEGO_HEIGHT/2, -LEGO_LENGTH/2> rotate y*90}
	  object { roundbox(r2-LEGO_HEIGHT, LEGO_HEIGHT*2/3, LEGO_LENGTH) translate <-r2+LEGO_HEIGHT/2, -(r1-r2), -LEGO_LENGTH/2> rotate y*90}
	  

	  intersection {
	    object { roundbox (2*LEGO_LENGTH, LEGO_HEIGHT, 2*LEGO_LENGTH)
	      translate <-LEGO_LENGTH/sqrt(2), -(r1-r2), -LEGO_LENGTH/sqrt(2)>
	      rotate y*135
	      translate z*l/2
	    }
	    box {
	      <-3*LEGO_LENGTH, 0,   0>
	      <3,-3,3>*LEGO_LENGTH
	    }
	  }
	}
	union {
	  intersection {
	    object { h_invroundbox (LEGO_HEIGHT-LEGO_CEILING_HEIGHT, 3*LEGO_LENGTH, 3*LEGO_LENGTH)
	      rotate z*90
	      scale <1,1,-1>
	      translate <LEGO_LENGTH/sqrt(2)-LEGO_WALL_WIDTH, -(r1-r2), LEGO_LENGTH/sqrt(2)-LEGO_WALL_WIDTH>
	      rotate -y*45
	      translate z*l/2
	    }
	    box {
	      <-3*LEGO_LENGTH, 0,   0>
	      <3,-3,3>*LEGO_LENGTH
	    }
	  }
	  object { h_invroundbox (LEGO_HEIGHT-LEGO_CEILING_HEIGHT+EXTRA, 3*LEGO_LENGTH-2*LEGO_WALL_WIDTH, LEGO_LENGTH-2*LEGO_WALL_WIDTH)
	    rotate z*90
	    translate <1.5*LEGO_LENGTH-LEGO_WALL_WIDTH, -(r1-r2), -LEGO_LENGTH/2+LEGO_WALL_WIDTH>
	  }
	  object { h_invroundbox (LEGO_HEIGHT-LEGO_CEILING_HEIGHT-EXTRA, LEGO_LENGTH-2*LEGO_WALL_WIDTH, 1.5*LEGO_LENGTH-LEGO_WALL_WIDTH)
	    rotate z*90
	    translate <LEGO_LENGTH/2-LEGO_WALL_WIDTH, -(r1-r2), 0>
	  }
	}
      }
      object { roundbox(LEGO_LENGTH*2, LEGO_HEIGHT-EXTRA, LEGO_CYLINDER_WALL_WIDTH)
	translate <-LEGO_LENGTH, -(r1-r2), -LEGO_CYLINDER_WALL_WIDTH/2> }
      object { roundbox(LEGO_CYLINDER_WALL_WIDTH, LEGO_HEIGHT-EXTRA, LEGO_LENGTH)
	translate <-LEGO_CYLINDER_WALL_WIDTH/2, -(r1-r2), 0> }
      object { roundcylinder(LEGO_HEIGHT-EXTRA, LEGO_CYLINDER_RADIUS)
	rotate z*90 translate <LEGO_LENGTH, -(r1-r2), 0> }
      object { roundcylinder(LEGO_HEIGHT-EXTRA, LEGO_CYLINDER_RADIUS)
	rotate z*90 translate <-LEGO_LENGTH, -(r1-r2), 0> }
      object { roundcylinder(LEGO_HEIGHT-EXTRA, LEGO_CYLINDER_RADIUS)
	rotate z*90 translate <0, -(r1-r2), LEGO_LENGTH> }
      
      object { roundcylinder(LEGO_KNOB_HEIGHT, LEGO_KNOB_RADIUS)
	rotate z*90
	translate y*(-(r1-r2)+LEGO_HEIGHT)
      }
      object { roundcylinder(LEGO_HEIGHT+LEGO_LENGTH/2, LEGO_LENGTH/2)
	rotate -z*90
	translate y*(-(r1-r2)+LEGO_HEIGHT-EXTRA)
      }
    }
    
    object { invroundcylinder(LEGO_HEIGHT-EXTRA, LEGO_KNOB_RADIUS)
      rotate z*90 translate <LEGO_LENGTH, -(r1-r2), 0> }
    object { invroundcylinder(LEGO_HEIGHT-EXTRA, LEGO_KNOB_RADIUS)
      rotate z*90 translate <-LEGO_LENGTH, -(r1-r2), 0> }
    object { invroundcylinder(LEGO_HEIGHT-EXTRA, LEGO_KNOB_RADIUS)
      rotate z*90 translate <0, -(r1-r2), LEGO_LENGTH> }

    object {
      __IAxle
      translate x*LEGO_LENGTH
      rotate -z*90
      translate -y*(LEGO_STEERINGWHEEL_TORUS_RADIUS+LEGO_CEILING_HEIGHT)
    }
    object {
      __INV_ROUND_AXLE
      rotate -z*90
      translate -y*(r1-r2+LEGO_LENGTH/2)
    }
    object {
      h_invroundcylinder(LEGO_KNOB_HEIGHT+LEGO_CEILING_HEIGHT+2*EXTRA, LEGO_KNOB_INNER_RADIUS)      
      rotate -z*90
      translate y*(-(r1-r2)+LEGO_HEIGHT+LEGO_KNOB_HEIGHT)
    }

    translate y*(LEGO_STEERINGWHEEL_TORUS_RADIUS + LEGO_CEILING_HEIGHT)
    
    texture { LegoBlack }
    rotate y*180
  }
  

  
  
  #declare wheel24x43a = union {
    object { roundtube(3*LEGO_LENGTH, 43/2, 43/2 - 1.2) }
    difference { 
      cylinder { <LEGO_LENGTH-1.4, 0, 0> <LEGO_LENGTH*2+1.4,0,0> 43/2 - 0.6 }
      cone { 
	<LEGO_LENGTH-1.4-EXTRA, 0, 0> 43/2 - 1.2 
	<LEGO_LENGTH, 0, 0>     43/2-2.6 }
      cone { 
	<LEGO_LENGTH*2+1.4+EXTRA, 0, 0> 43/2 - 1.2 
	<LEGO_LENGTH*2, 0, 0>     43/2-2.6 }
      cylinder { <0,0,0> <LEGO_LENGTH*3,0,0> 43/2-2.6 }
    }	
    object { roundtube(2*l, 20/2, 20/2-1.2)
      translate x*LEGO_LENGTH/2 }
    difference {
      object { roundcylinder(1.2, 25/2) }
      cylinder{ <-EXTRA,0,0><2,0,0> 20/2 }
      translate x*(LEGO_LENGTH/2+2)
    }
    intersection {
      difference {
	union {
	  object { roundbox(2*LEGO_LENGTH, 5, 20)
	    translate <0, -2.5, -10> }
	  object { roundcylinder (LEGO_LENGTH*2, LEGO_LENGTH/2) }
	}
	object { invroundbox(2*LEGO_LENGTH, 1, 22)
	  translate <0, -0.5, -11> }
	object { __IAxle scale <2,1,1> }
	object { __INV_ROUND_AXLE scale <-1,1,1> }
	object { __INV_ROUND_AXLE translate x*LEGO_LENGTH*2 }
      }
      cylinder{ <-EXTRA,0,0><2*LEGO_LENGTH+EXTRA, 0, 0> 20/2-0.6}
      translate x*LEGO_LENGTH/2
    }
    
    intersection {
      difference {
	union {
	  #local i=0;
	  #while (i<6)	    
	    difference {
	      union {
		object { roundbox(LEGO_LENGTH-0.2, 1, 43)
		  translate <LEGO_LENGTH/2+0.2, -0.5, -43/2> 
		  rotate (360/12/2+360/12*i)*x }
		
		intersection {
		  box{ 0     <2.8,  20, -43/2>       rotate -x*360/12/2}
		  box{ EXTRA <2.8, -20, -43/2>+EXTRA rotate  x*360/12/2}
		  rotate x*360/6*i
		  translate x*(LEGO_LENGTH/2+0.4)
		}
		object { h_roundtube(LEGO_LENGTH/2, 6.4/2, LEGO_KNOB_RADIUS)
		  translate <0.8, 0, -2*LEGO_LENGTH> 
		  rotate x*360/6*i }
	      }
	      cylinder { 0 x*LEGO_LENGTH*3 LEGO_KNOB_RADIUS+EXTRA 
		translate -z*2*LEGO_LENGTH 
		rotate x*360/6*i }
	    }
	    
	    #local i=i+1;
	  #end
	}
	cylinder { 0 x*LEGO_LENGTH*2 20/2-0.6 }
      }
      cylinder { 0 x*LEGO_LENGTH*2 43/2-0.6 }
    }
  }
  
  

  #declare wheel24x43cc = difference {
    box { 0 <83/2, LEGO_LENGTH*4, 2*LEGO_LENGTH> }
    object { inv_roundedge(4*LEGO_LENGTH, 90)
      rotate -z*90
      translate <83/2, 4*LEGO_LENGTH, 0> }
    
    translate -y*LEGO_LENGTH*2
    rotate x*1.9 
    rotate y*360/56/2
  }

  
  #declare wheel24x43c = difference {
    intersection {
      object { wheel24x43cc }
      object { wheel24x43cc scale <1,1,-1> }
      box { <83/2-LEGO_LENGTH*2, 0, -LEGO_LENGTH> <83/2+LEGO_LENGTH, -1.5*LEGO_LENGTH, LEGO_LENGTH> }
    }
    object { inv_roundedge(4*LEGO_LENGTH, 91.9)
      translate x*(83/2-LEGO_LENGTH*2)
      rotate y*360/56/2 }
    object { inv_roundedge(4*LEGO_LENGTH, 91.9)
      translate x*(83/2-LEGO_LENGTH*2)
      rotate y*360/56/2 
      scale <1,1,-1> }
    rotate z*90
  }
  
  #declare wheel24x43b = intersection {
    union {
      #local i=0;
      #while(i<28) 
	object { wheel24x43c rotate x*360/28*i }
	object { wheel24x43c 
	  scale <-1,1,1>
	  translate x*3*LEGO_LENGTH
	  rotate x*(360/28*i+360/56) }
	#local i=i+1;
      #end
      object { roundcylinder(3*LEGO_LENGTH, 43/2+13.2) }
    }
    object { roundtube (3*LEGO_LENGTH+2*EXTRA, 83/2+EXTRA, 43/2) 
      translate -x*EXTRA }
  }
  
  
  #declare wheel24x43 = union {
    object { wheel24x43a }
    object { wheel24x43b texture { LegoBlack } }
  }

#end